// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
using SimpleJSON;
using System;
using System.Diagnostics.CodeAnalysis;
using System.Text.RegularExpressions;

static class EntryPointUtils
{
    public static Regex NaturalisBMRegex { get; } = new Regex($@"^plugin#\d+_{Constant.NATURALIS}.{Constant.BOOTY_MAGIC}", RegexOptions.Compiled);
    public static Regex NaturalisTMRegex { get; } = new Regex($@"^plugin#\d+_{Constant.NATURALIS}.{Constant.TITTY_MAGIC}", RegexOptions.Compiled);

    public static float GetFloatValue(JSONClass jc, string key, float defaultValue)
    {
        if(!jc.HasKey(key))
        {
            return defaultValue;
        }

        float value = jc[key].AsFloat;
        jc.Remove(key);
        return value;
    }

    public static bool GetBoolValue(JSONClass jc, string key, bool defaultValue)
    {
        if(!jc.HasKey(key))
        {
            return defaultValue;
        }

        bool value = jc[key].AsBool;
        jc.Remove(key);
        return value;
    }

    public static string GetPluginId(string pluginStorableName) =>
        pluginStorableName.Substring(0, pluginStorableName.IndexOf('_'));

    public static string GetPackageId(MVRPluginManager manager, string pluginUid)
    {
        string filename = manager.GetJSON()["plugins"][pluginUid].Value;
        int idx = filename.IndexOf(":/", StringComparison.Ordinal);
        return idx >= 0 ? filename.Substring(0, idx) : null;
    }

    public static void AddNaturalis(MVRPluginManager manager, string packagePrefix)
    {
        var plugin = manager.CreatePlugin();
        plugin.pluginURLJSON.val = $"{packagePrefix}Custom/Scripts/{Constant.EVERLASTER}/{Constant.NATURALIS}/{Constant.NATURALIS}.dll";
    }
}

static class JSONStorableExtensions
{
    public static JSONStorable FirstOwnAtomStorableByRegex(this JSONStorable self, Regex regex)
    {
        var storableIds = self.containingAtom.GetStorableIDs();
        foreach(string id in storableIds)
        {
            if(regex.IsMatch(id))
            {
                var storable = self.containingAtom.GetStorableByID(id);
                if(storable && storable.containingAtom && storable != self)
                {
                    return storable;
                }
            }
        }

        return null;
    }
}

[SuppressMessage("ReSharper", "InconsistentNaming")]
static class Constant
{
    public const string EVERLASTER = "everlaster";
    public const string NATURALIS = "Naturalis";
    public const string BOOTY_MAGIC = "BootyMagic";
    public const string TITTY_MAGIC = "TittyMagic";
    [SuppressMessage("ReSharper", "InconsistentNaming")]
    const string INTERNAL = "internal_";
    public readonly static string INTERNAL_IS_CALIBRATING = $"{INTERNAL}isCalibrating";
    public readonly static string INTERNAL_IS_MIGRATION_SCRIPT = $"{INTERNAL}isMigrationScript";
}
