using NAT.Common;
using NAT.Common.Components;
using System;
using System.Collections;
using UnityEngine;

namespace NAT
{
    sealed class SharedUIModManager : UIModManagerBase
    {
        readonly Bootstrap _bootstrap;
        readonly UIMod _femaleMorphsMod;
        readonly UIMod _maleMorphsMod;
        readonly UIMod _skinMaterials2Mod;
        float _morphsTabVisibleDuration;

        internal bool IsMorphsTabVisibleFor1Sec()
        {
            if(!initialized)
            {
                return false;
            }

            _morphsTabVisibleDuration = _femaleMorphsMod.IsVisible() || _maleMorphsMod.IsVisible()
                ? _morphsTabVisibleDuration + Time.deltaTime
                : 0f;
            return _morphsTabVisibleDuration > 1f;
        }

        internal SharedUIModManager(Bootstrap bootstrap)
        {
            _bootstrap = bootstrap;
            _femaleMorphsMod = new UIMod(bootstrap, "Female Morphs", CreateFemaleMorphsPauseToggleCo, true);
            _maleMorphsMod = new UIMod(bootstrap, "Male Morphs", CreateMaleMorphsPauseToggleCo, true);
            _skinMaterials2Mod = new UIMod(bootstrap, "Skin Materials 2", MoveSkinMaterialsUIButtonGroupCo);

            uiMods[Constant.FEMALE_MORPHS] = _femaleMorphsMod;
            uiMods[Constant.MALE_MORPHS] = _maleMorphsMod;
            uiMods[Constant.SKIN_MATERIALS_2_MOVE_BTN_GROUP] = _skinMaterials2Mod;

            foreach(var pair in uiMods)
            {
                pair.Value.DeferApply();
            }

            if(!bootstrap.IsEnabledAndActive())
            {
                Disable();
            }

            initialized = true;
        }

        IEnumerator CreateFemaleMorphsPauseToggleCo(UIMod uiMod)
        {
            while(_bootstrap.Done != true)
            {
                yield return null;
            }

            try
            {
                _bootstrap.CreatePauseToggle(uiMod, _bootstrap.PauseFemaleMorphUpdatesJsb);
                uiMod.AddTargetUIEventsListener();
            }
            catch(Exception e)
            {
                Loggr.Error($"CreateFemaleMorphsPauseToggleCo: {e}");
            }
        }

        IEnumerator CreateMaleMorphsPauseToggleCo(UIMod uiMod)
        {
            while(_bootstrap.Done != true)
            {
                yield return null;
            }

            try
            {
                _bootstrap.CreatePauseToggle(uiMod, _bootstrap.PauseMaleMorphUpdatesJsb);
                uiMod.AddTargetUIEventsListener();
            }
            catch(Exception e)
            {
                Loggr.Error($"CreateMaleMorphsPauseToggleCo: {e}");
            }
        }

        IEnumerator MoveSkinMaterialsUIButtonGroupCo(UIMod uiMod)
        {
            while(_bootstrap.Done != true)
            {
                yield return null;
            }

            var rightSide = uiMod.Target.Find("RightSide");
            uiMod.RepositionTransform(rightSide.Find("SavePanel"), new Vector2(-440, 70), new Vector2(-440, 670));
            uiMod.RepositionTransform(rightSide.Find("RestorePanel"), new Vector2(-230, 70), new Vector2(-230, 670));
            uiMod.RepositionTransform(rightSide.Find("Reset"), new Vector2(-70, 70), new Vector2(-70, 670));

            if(_bootstrap.Person.HasDummySkin() || !_bootstrap.IsEnabledAndActive())
            {
                uiMod.Disable();
            }
        }

        internal void SetSkinMaterials2ModEnabled(bool value)
        {
            if(value)
            {
                if(!_bootstrap.Person.HasDummySkin())
                {
                    _skinMaterials2Mod.Enable();
                }
            }
            else
            {
                _skinMaterials2Mod.Disable();
            }
        }

        bool _enableSkinMaterials2Mod;

        internal void SetEnabled(bool enableSkinMaterials2Mod = false)
        {
            _enableSkinMaterials2Mod = enableSkinMaterials2Mod;
            Enable();
        }

        internal override void Enable()
        {
            _femaleMorphsMod.Enable();
            _maleMorphsMod.Enable();

            if(_enableSkinMaterials2Mod && !_bootstrap.Person.HasDummySkin())
            {
                _skinMaterials2Mod.Enable();
            }
        }
    }
}
