using NAT.Common;
using NAT.Common.Handlers;
using NAT.Common.Models;
using NAT.Common.Storables;
using System.Collections.Generic;
using UnityEngine;
using static NAT.Direction;

namespace NAT.BM.Handlers
{
    sealed class GluteForceMorphHandler : ForceMorphHandler
    {
        internal GluteForceMorphHandler(ScriptBase script) : base(script)
        {
            baseJsf = NewMultiplierJsf("Force Morphing Base Multiplier");
            upJsf = NewMultiplierJsf("Force Morphing Up Multiplier");
            downJsf = NewMultiplierJsf("Force Morphing Down Multiplier");
            depthOutJsf = NewMultiplierJsf("Force Morphing Depth Out Multiplier");
            depthInJsf = NewMultiplierJsf("Force Morphing Depth In Multiplier");
            sidewaysInJsf = NewMultiplierJsf("Force Morphing Sideways In Multiplier");
            sidewaysOutJsf = NewMultiplierJsf("Force Morphing Sideways Out Multiplier");
            LoadSettings();
        }

        protected override void OnMultiplierValueChanged(JSONStorableFloat jsf)
        {
            if(Script.IsLdpUpdateOn)
            {
                Script.SetCalibrationPending((StorableFloat) jsf);
            }
        }

        protected override void LoadSettings()
        {
            morphSets[UP_L] = UpForceMorphs("L");
            morphSets[UP_R] = UpForceMorphs("R");
            morphSets[DOWN_L] = DownForceMorphs("L");
            morphSets[DOWN_R] = DownForceMorphs("R");
            morphSets[BACK_L] = DepthInForceMorphs("L");
            morphSets[BACK_R] = DepthInForceMorphs("R");
            morphSets[FORWARD_L] = DepthOutForceMorphs("L");
            morphSets[FORWARD_R] = DepthOutForceMorphs("R");
            morphSets[LEFT_L] = LeftForceLeftMorphs();
            morphSets[LEFT_R] = LeftForceRightMorphs();
            morphSets[RIGHT_L] = RightForceLeftMorphs();
            morphSets[RIGHT_R] = RightForceRightMorphs();

            var directions = new List<string>
            {
                UP,
                DOWN,
                BACK,
                FORWARD,
                LEFT_L,
                RIGHT_L,
                LEFT_R,
                RIGHT_R,
            };
            directionJssc = new JSONStorableStringChooser("direction", directions, directions[0], "Direction");
        }

        ConfigurableMorph NewConfigurableMorph(string morphName, bool isNegative, float softMultiplier, float massMultiplier) =>
            new ConfigurableMorph(
                Person.GetGluteMorph(morphName),
                isNegative,
                new JSONStorableFloat("softMultiplier", softMultiplier, -3.00f, 3.00f),
                new JSONStorableFloat("massMultiplier", massMultiplier, -3.00f, 3.00f)
            );

        List<ConfigurableMorph> UpForceMorphs(string side)
        {
            const string prefix = MorphPrefix.UP;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Y {side}", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute UpDown {side}", true, -1.60f, 0.60f),
                NewConfigurableMorph($"{prefix} Glute Move Up {side}", false, 1.80f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Upper Spread {side}", false, 0.50f, -0.25f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", false, 0.60f, 0.40f),
            };
        }

        List<ConfigurableMorph> DownForceMorphs(string side)
        {
            const string prefix = MorphPrefix.DN;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Y {side}", true, -0.50f, -1.00f),
                NewConfigurableMorph($"{prefix} Glute UpDown {side}", false, 0.00f, 0.00f),
                NewConfigurableMorph($"{prefix} Glute Move Down {side}", true, -0.15f, -0.30f),
                NewConfigurableMorph($"{prefix} Glute Height Lower {side}", true, -0.20f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Line {side}", false, 0.30f, 0.60f),
            };
        }

        List<ConfigurableMorph> DepthInForceMorphs(string side)
        {
            const string prefix = MorphPrefix.BK;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Scale Z {side}", true, -0.90f, -1.20f),
                NewConfigurableMorph($"{prefix} Glute Height {side}", false, 1.00f, 1.80f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", false, 1.50f, 2.85f),
            };
        }

        List<ConfigurableMorph> DepthOutForceMorphs(string side)
        {
            const string prefix = MorphPrefix.FW;
            return new List<ConfigurableMorph>
            {

                NewConfigurableMorph($"{prefix} Glute Scale Z {side}", false, 0.80f, 1.60f),
                NewConfigurableMorph($"{prefix} Glute Height {side}", true, -0.36f, -0.48f),
                NewConfigurableMorph($"{prefix} Glute Width {side}", true, -0.50f, -0.67f),
            };
        }

        List<ConfigurableMorph> LeftForceLeftMorphs()
        {
            const string prefix = MorphPrefix.LT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z L", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Move Out L", false, 1.00f, -0.20f),
            };
        }

        List<ConfigurableMorph> LeftForceRightMorphs()
        {
            const string prefix = MorphPrefix.LT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z R", true, -0.45f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Move In R", false, 1.10f, -0.40f),
            };
        }

        List<ConfigurableMorph> RightForceLeftMorphs()
        {
            const string prefix = MorphPrefix.RT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z L", true, -0.45f, -0.40f),
                NewConfigurableMorph($"{prefix} Glute Move In L", false, 1.10f, -0.40f),
            };
        }

        List<ConfigurableMorph> RightForceRightMorphs()
        {
            const string prefix = MorphPrefix.RT;
            return new List<ConfigurableMorph>
            {
                NewConfigurableMorph($"{prefix} Glute Rotate Z R", true, -0.25f, -0.50f),
                NewConfigurableMorph($"{prefix} Glute Move Out R", false, 1.00f, -0.20f),
            };
        }

        internal override void PreRegisterParams()
        {
            Script.PreRegisterMorphsParam(baseJsf);
            Script.PreRegisterMorphsParam(Script.upDownAngleOffsetJsf);
            Script.PreRegisterMorphsParam(Script.depthOffsetJsf);
            Script.PreRegisterMorphsParam(Script.leftRightAngleOffsetJsf);
            Script.PreRegisterMorphsParam(upJsf);
            Script.PreRegisterMorphsParam(downJsf);
            Script.PreRegisterMorphsParam(depthOutJsf);
            Script.PreRegisterMorphsParam(depthInJsf);
            Script.PreRegisterMorphsParam(sidewaysInJsf);
            Script.PreRegisterMorphsParam(sidewaysOutJsf);
        }

        float _mass;
        float _softness;

        float _upMultiplier;
        float _downMultiplier;
        float _depthInMultiplier;
        float _depthOutMultiplier;
        float _sidewaysInMultiplier;
        float _sidewaysOutMultiplier;

        static float SoftnessMultiplier(float x) => Calc.InverseSmoothStep(x, 0.80f, 0.03f);

        internal override void SetMultipliers()
        {
            _mass = Script.jointPhysicsHandler.normalizedRealMass;
            _softness = SoftnessMultiplier(Script.softness);
            SetUpDownMultipliers();
            SetDepthInMultiplier();
            SetDepthOutMultiplier();
            SetSidewaysMultipliers();
        }

        void SetUpDownMultipliers()
        {
            upLimit = 30f;
            upDownNormalizer = 50 * Mathf.Lerp(1, 1.20f, _mass);
            float softnessMultiplier = Mathf.Lerp(0.50f, 1.25f, _softness);
            _upMultiplier = softnessMultiplier * 1.50f * Mathf.Lerp(1, 1.80f, _mass);
            _downMultiplier = softnessMultiplier * 1.80f * Mathf.Lerp(1, 1.80f, _mass);
        }

        void SetDepthInMultiplier()
        {
            depthInNormalizer = 0.045f * Mathf.Lerp(1, 3.00f, _mass);
            float softnessMultiplier = Mathf.Lerp(0.50f, 1.00f, _softness);
            _depthInMultiplier = softnessMultiplier * 4.40f * Mathf.Lerp(1, 0.50f, _mass);
        }

        void SetDepthOutMultiplier()
        {
            depthOutLimit = 0.020f;
            depthOutNormalizer = 0.045f * Mathf.Lerp(1, 3.00f, _mass);
            float softnessMultiplier = Mathf.Lerp(0.65f, 1.125f, _softness);
            _depthOutMultiplier = softnessMultiplier * 2.20f * Mathf.Lerp(1, 1.50f, _mass);
        }

        void SetSidewaysMultipliers()
        {
            sidewaysInLimit = 30f;
            sidewaysOutLimit = Mathf.Lerp(30f, 40f, _mass);
            sidewaysNormalizer = 50 * Mathf.Lerp(1, 1.20f, _mass);
            float softnessMultiplier = Mathf.Lerp(0.50f, 1.25f, _softness);
            _sidewaysInMultiplier = softnessMultiplier * 1.10f * Mathf.Lerp(1, 1.80f, _mass);
            _sidewaysOutMultiplier = softnessMultiplier * 0.80f * Mathf.Lerp(1, 1.80f, _mass);
        }

        internal override void OnFixedUpdate()
        {
            AdjustUpMorphs();
            AdjustDownMorphs();
            AdjustDepthOutMorphs();
            AdjustDepthInMorphs();
            AdjustSidewaysMorphs();
        }

        static float UpDownForceEffectCurve(float x) => Calc.Exponential1(x, 1.60f, 2.26f, 1.23f, 0.96f);

        float UpEffect(float angle) =>
            _upMultiplier *
            UserUpMultiplier() *
            UpDownForceEffectCurve(Mathf.Min(Mathf.Abs(angle), upLimit) / upDownNormalizer);

        void AdjustUpMorphs()
        {
            if(leftTarget.angleY <= 0)
            {
                UpdateMorphs(UP_L, UpEffect(leftTarget.angleY));
            }
            else
            {
                ResetMorphs(UP_L);
            }

            if(rightTarget.angleY <= 0)
            {
                UpdateMorphs(UP_R, UpEffect(rightTarget.angleY));
            }
            else
            {
                ResetMorphs(UP_R);
            }
        }

        float DownEffect(float angle) => _downMultiplier *
            UserDownMultiplier() *
            UpDownForceEffectCurve(Mathf.Abs(angle) / upDownNormalizer);

        void AdjustDownMorphs()
        {
            if(leftTarget.angleY <= 0)
            {
                ResetMorphs(DOWN_L);
            }
            else
            {
                UpdateMorphs(DOWN_L, DownEffect(leftTarget.angleY));
            }

            if(rightTarget.angleY <= 0)
            {
                ResetMorphs(DOWN_R);
            }
            else
            {
                UpdateMorphs(DOWN_R, DownEffect(rightTarget.angleY));
            }
        }

        static float DepthForceEffectCurve(float x) => Calc.Exponential1(x, 1.60f, 2.26f, 1.23f, 0.96f);

        float DepthInEffect(float depthDiff) =>
            _depthInMultiplier *
            UserDepthInMultiplier() *
            DepthForceEffectCurve(Mathf.Abs(depthDiff) / depthInNormalizer);

        void AdjustDepthInMorphs()
        {
            if(leftTarget.depthDiff >= 0)
            {
                ResetMorphs(BACK_L);
            }
            else
            {
                UpdateMorphs(BACK_L, DepthInEffect(leftTarget.depthDiff));
            }

            if(rightTarget.depthDiff >= 0)
            {
                ResetMorphs(BACK_R);
            }
            else
            {
                UpdateMorphs(BACK_R, DepthInEffect(rightTarget.depthDiff));
            }
        }

        float DepthOutEffect(float depthDiff) =>
            _depthOutMultiplier *
            UserDepthOutMultiplier() *
            DepthForceEffectCurve(Mathf.Min(Mathf.Abs(depthDiff), depthOutLimit) / depthOutNormalizer);

        void AdjustDepthOutMorphs()
        {
            if(leftTarget.depthDiff >= 0)
            {
                UpdateMorphs(FORWARD_L, DepthOutEffect(leftTarget.depthDiff));
            }
            else
            {
                ResetMorphs(FORWARD_L);
            }

            if(rightTarget.depthDiff >= 0)
            {
                UpdateMorphs(FORWARD_R, DepthOutEffect(rightTarget.depthDiff));
            }
            else
            {
                ResetMorphs(FORWARD_R);
            }
        }

        static float SidewaysForceEffectCurve(float x) => Calc.Exponential1(x, 1.60f, 2.26f, 1.23f, 0.96f);

        float SidewaysInEffect(float angle) =>
            _sidewaysInMultiplier *
            UserSidewaysInMultiplier() *
            SidewaysForceEffectCurve(Mathf.Min(Mathf.Abs(angle), sidewaysInLimit) / sidewaysNormalizer);

        float SidewaysOutEffect(float angle) =>
            _sidewaysOutMultiplier *
            UserSidewaysOutMultiplier() *
            SidewaysForceEffectCurve(Mathf.Min(Mathf.Abs(angle), sidewaysOutLimit) / sidewaysNormalizer);

        void AdjustSidewaysMorphs()
        {
            if(leftTarget.angleX <= 0)
            {
                ResetMorphs(LEFT_L);
                UpdateMorphs(RIGHT_L, SidewaysInEffect(leftTarget.angleX));
            }
            else
            {
                ResetMorphs(RIGHT_L);
                UpdateMorphs(LEFT_L, SidewaysOutEffect(leftTarget.angleX));
            }

            if(rightTarget.angleX <= 0)
            {
                ResetMorphs(LEFT_R);
                UpdateMorphs(RIGHT_R, SidewaysOutEffect(rightTarget.angleX));
            }
            else
            {
                ResetMorphs(RIGHT_R);
                UpdateMorphs(LEFT_R, SidewaysInEffect(rightTarget.angleX));
            }
        }

        void UpdateMorphs(string direction, float effect)
        {
            float halfEffect = 0.5f * effect;
            foreach(var configurableMorph in morphSets[direction])
            {
                configurableMorph.SetValue(halfEffect, _mass, 1);
            }
        }

        internal override void ResetAll()
        {
            if(morphSets == null)
            {
                return;
            }

            foreach(var pair in morphSets)
            {
                foreach(var configurableMorph in pair.Value)
                {
                    configurableMorph.Reset();
                }
            }
        }

        void ResetMorphs(string direction)
        {
            foreach(var configurableMorph in morphSets[direction])
            {
                configurableMorph.Reset();
            }
        }

        internal override void Disable()
        {
            ResetAll();
        }
    }
}
