using NAT.Common;
using NAT.Common.Handlers;
using UnityEngine;

namespace NAT.BM.Handlers
{
    sealed class GluteFrictionHandler : FrictionHandler
    {
        readonly SoftPhysicsHandler _softPhysicsHandler;

        internal GluteFrictionHandler(ScriptBase script) : base(script)
        {
            _softPhysicsHandler = Script.softPhysicsHandler;
            SetupStorables();
            LoadSettings();
        }

        protected override void LoadSettings()
        {
            if(!IsEnabled)
            {
                return;
            }

            _softPhysicsHandler.fatColliderRadius.SetFrictionCalculators(
                FatColliderRadiusCalculator(),
                FatColliderRadiusCalculator()
            );
        }

        protected override void UpdateSoftColliders(float friction)
        {
            if(!_softPhysicsHandler.softPhysicsOn)
            {
                return;
            }

            if(AdaptiveFrictionJsb.val)
            {
                _softPhysicsHandler.fatColliderRadius.UpdateInverseFrictionValue(
                    1 - Mathf.InverseLerp(0, DrySkinFrictionJsf.val, friction),
                    _softPhysicsHandler.fatColliderRadius.usesRealMass
                        ? Script.jointPhysicsHandler.normalizedRealMass
                        : Script.jointPhysicsHandler.normalizedMass,
                    Script.softness
                );
            }
            else
            {
                _softPhysicsHandler.fatColliderRadius.ResetInverseFrictionValue();
            }

            _softPhysicsHandler.fatColliderFriction.Left.offsetJsf.val = friction;
        }

        /* Maximum friction that a collider can have, drops off dynamically with distance from collider's normal position */
        internal override void CalculateFriction()
        {
            if(!IsEnabled || !Script.IsEnabledNullSafe() || glossJsf == null || specularBumpinessJsf == null)
            {
                return;
            }

            float maxHardColliderFriction;
            if(!AdaptiveFrictionJsb.val)
            {
                float baseMax = _softPhysicsHandler.softPhysicsOn ? 0.60f : 0.80f;
                maxHardColliderFriction = Mathf.Clamp(baseMax, 0.15f, 1.00f);
                FrictionValueJsf.valNoCallback = Mathf.Clamp(0.80f + FrictionOffsetJsf.val, 0.00f, 1.00f);
            }
            else
            {
                float normalizedLinearGloss = Mathf.InverseLerp(3.000f, 7.500f, glossJsf.val);
                float normalizedSpecBump = Mathf.InverseLerp(0.200f, 3.000f, specularBumpinessJsf.val);

                /* Gloss based multiplier for how much effect bumpiness has on friction */
                float bumpinessEffectMultiplier = Calc.InverseSmoothStep(normalizedLinearGloss, 0.50f, 0.90f);

                /* Inverse of gloss determines the minimum value for friction along a curve. */
                float minFriction = DrySkinFrictionJsf.val * (1 - Calc.InverseSmoothStep(normalizedLinearGloss, 0.50f, 0.90f));

                /* Hard colliders */
                {
                    float specBumpComponent = 0.350f * bumpinessEffectMultiplier * normalizedSpecBump;
                    float friction = 0.80f * Mathf.Lerp(minFriction, 1.000f, specBumpComponent);
                    maxHardColliderFriction = Mathf.Clamp(friction + FrictionOffsetJsf.val, 0.15f, 1.00f);
                }

                /* Soft colliders */
                {
                    float specBumpComponent = 0.350f * bumpinessEffectMultiplier * normalizedSpecBump;
                    float friction = Mathf.Lerp(minFriction, 1.000f, specBumpComponent);
                    FrictionValueJsf.valNoCallback = Mathf.Clamp(friction + FrictionOffsetJsf.val, 0.00f, 1.00f);
                }
            }

            Script.hardColliderHandler.maxHardColliderFriction = maxHardColliderFriction;
            UpdateSoftColliders(FrictionValueJsf.val);
        }
    }
}
