using ColliderEditor.Extensions;
using System;
using System.Collections.Generic;
using UnityEngine;

namespace ColliderEditor.Models
{
    abstract class ModelBase<T> where T : Component
    {
        public List<Group> Groups { get; }
        public bool Highlighted { get; set; }
        internal ModelBase<T> mirror;

        public IModel MirrorModel => mirror as IModel;

        public string Id { get; }
        public string Label { get; }

        // ReSharper disable once UnusedAutoPropertyAccessor.Global
        public bool Selected { get; set; }

        protected ModelBase(T component, string label)
        {
            if(!component)
            {
                throw new ArgumentNullException(nameof(component));
            }

            if(string.IsNullOrEmpty(label))
            {
                throw new ArgumentException("message", nameof(label));
            }

            Groups = new List<Group>();
            Id = component.Uuid();
            Label = label;
        }

        public abstract void SyncPreviews();

        public override string ToString() => Id;
    }
}
