using ColliderEditor.Extensions;
using ColliderEditor.Models;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace ColliderEditor
{
    sealed class SelectablesList
    {
        internal readonly List<Group> groups;
        internal readonly List<IModel> all = new List<IModel>();
        internal readonly Dictionary<string, IModel> byUuid = new Dictionary<string, IModel>();

        internal SelectablesList(
            IList<Collider> leftColliders,
            IList<Collider> rightColliders,
            ColliderPreviewConfig config,
            List<Group> groups
        )
        {
            if(leftColliders.Count != rightColliders.Count)
            {
                throw new System.ArgumentException($"leftColliders count {leftColliders.Count} != rightColliders count {rightColliders.Count})");
            }

            var materialHelper = new MaterialHelper();
            this.groups = groups;

            for(int i = 0; i < leftColliders.Count; i++)
            {
                var leftCollider = leftColliders[i];
                var rightCollider = rightColliders[i];
                var material = materialHelper.GetNextMaterial(leftCollider.Uuid().GetHashCode());

                var leftModel = ColliderModel.CreateTyped(leftCollider, config, material);
                var rightModel = ColliderModel.CreateTyped(rightCollider, config, material);

                leftModel.mirror = rightModel;
                rightModel.mirror = leftModel;

                leftModel.Groups.AddRange(groups.Where(group => group.Test(leftModel.Collider.name)));
                rightModel.Groups.AddRange(groups.Where(group => group.Test(rightModel.Collider.name)));

                all.Add(leftModel);
                all.Add(rightModel);

                byUuid[leftModel.Collider.name] = leftModel;
                byUuid[rightModel.Collider.name] = rightModel;
            }
        }
    }
}
