using System;

namespace NAT.Common.Components
{
    sealed class CalibrationState
    {
        readonly Person _person;
        bool? _collisionEnabled;
        string _poseName;
        PresetParams? _posePresetParams;

        internal CalibrationState(Person person)
        {
            _person = person;
        }

        internal void Apply()
        {
            DisableCollision();
            LoadCalibrationPose();
        }

        void DisableCollision()
        {
            if(_collisionEnabled == null)
            {
                _collisionEnabled = _person.CollisionEnabledJsb.val;
                _person.CollisionEnabledJsb.val = false;
            }
        }

        void LoadCalibrationPose()
        {
            _posePresetParams = _person.GetPosePresetParams();
            _poseName = $"calibration_{Guid.NewGuid().ToString().Substring(0, 7)}";
            _person.SavePosePreset(_poseName, storeRootNodeInPose: true);
            string calibrationPosePresetName = _person.atom.mainController.GetBoolParamValue("detachControl")
                ? Constant.NO_ROOT_CALIBRATION_POSE
                : Constant.DEFAULT_CALIBRATION_POSE;
            _person.LoadPosePreset(calibrationPosePresetName, true);
        }

        internal void Restore()
        {
            RestorePose();
            RestoreCollision();
        }

        void RestorePose()
        {
            _person.LoadPosePreset(_poseName, false);
            _person.DeletePosePreset(_poseName);
            if(_posePresetParams != null)
            {
                _person.SetPosePresetParams(_posePresetParams.Value);
            }

            _poseName = null;
            _posePresetParams = null;
        }

        void RestoreCollision()
        {
            if(_collisionEnabled == true)
            {
                _person.CollisionEnabledJsb.val = true;
            }

            _collisionEnabled = null;
        }
    }
}
