namespace NAT.Common.Components
{
    sealed class FreeControllerState
    {
        readonly FreeControllerV3 _controller;
        FreeControllerV3.PositionState _positionState;
        FreeControllerV3.RotationState _rotationState;

        internal FreeControllerState(FreeControllerV3 controller)
        {
            _controller = controller;
        }

        internal void SetStateOn()
        {
            _positionState = _controller.currentPositionState;
            _rotationState = _controller.currentRotationState;
            _controller.currentPositionState = FreeControllerV3.PositionState.On;
            _controller.currentRotationState = FreeControllerV3.RotationState.On;
        }

        internal void Restore()
        {
            _controller.currentPositionState = _positionState;
            _controller.currentRotationState = _rotationState;
        }

        internal static bool IsControllerLinked(FreeControllerV3 controller) =>
            controller.currentPositionState == FreeControllerV3.PositionState.ParentLink ||
            controller.currentPositionState == FreeControllerV3.PositionState.PhysicsLink ||
            controller.currentRotationState == FreeControllerV3.RotationState.ParentLink ||
            controller.currentRotationState == FreeControllerV3.RotationState.PhysicsLink;
    }
}
