using NAT.Common.Handlers;
using System.Collections.Generic;

namespace NAT.Common.Components
{
    abstract class UIModManagerBase
    {
        protected ScriptBase script;
        protected FrictionHandler frictionHandler;
        protected readonly Dictionary<string, UIMod> uiMods = new Dictionary<string, UIMod>();
        protected bool initialized;
        internal bool skinMaterials2Modified;
        internal UIDynamicSlider drySkinFrictionSlider;

        internal virtual void SetFrictionUIVisibility()
        {
        }

        internal void SwitchToOwnFrictionUI()
        {
            ShowFrictionUI();
            script.OtherPlugin.uiModManager.HideFrictionUI();
        }

        internal void ShowFrictionUI()
        {
            if(!script.enabled || skinMaterials2Modified)
            {
                return;
            }

            if(frictionHandler.IsEnabled)
            {
                uiMods[Constant.SKIN_MATERIALS_2_FRICTION].Enable();
            }

            uiMods[Constant.SKIN_MATERIALS_2_ADD_BTN].Disable();

            skinMaterials2Modified = true;
        }

        internal void HideFrictionUI()
        {
            if(!script.enabled || !skinMaterials2Modified)
            {
                return;
            }

            uiMods[Constant.SKIN_MATERIALS_2_FRICTION].Disable();
            if(frictionHandler.IsEnabled)
            {
                uiMods[Constant.SKIN_MATERIALS_2_ADD_BTN].Enable();
            }

            skinMaterials2Modified = false;
        }

        internal virtual void Disable()
        {
            foreach(var pair in uiMods)
            {
                pair.Value.Disable();
            }
        }

        internal abstract void Enable();

        internal void Destroy()
        {
            foreach(var pair in uiMods)
            {
                pair.Value.Destroy();
            }
        }
    }
}
