using NAT.Common.Storables;

namespace NAT.Common.Handlers.Configs
{
    sealed class PhysicsCalculator
    {
        internal StorableFloat multiplierJsf;
        internal MultiplierConfig primaryConfig;
        internal MultiplierConfig secondaryConfig;

        internal float Calculate(float mass, float softness, bool usePrimary) =>
            Calculate(usePrimary ? primaryConfig : secondaryConfig, mass, softness);

        static float Calculate(MultiplierConfig config, float valueA, float valueB)
        {
            if(config == null || config.baseValue == 0)
            {
                return 0;
            }

            float baseValue = config.baseValue;
            return baseValue +
                baseValue * config.multiplierA * config.curveA(valueA) +
                baseValue * config.multiplierB * config.curveB(valueB);
        }

        internal float CalculateDynamic(float effect, float valueA, float valueB, bool usePrimary) =>
            CalculateDynamic(usePrimary ? primaryConfig : secondaryConfig, effect, valueA, valueB);

        float CalculateDynamic(MultiplierConfig config, float effect, float valueA, float valueB)
        {
            float multiplier = (multiplierJsf?.val ?? 1) * effect;
            return multiplier *
            (
                config.baseValue +
                config.multiplierA * config.curveA(valueA) +
                config.multiplierB * config.curveB(valueB)
            );
        }
    }
}
