using NAT.Common.Models;
using NAT.Common.Storables;

namespace NAT.Common.Handlers
{
    abstract class ForceHandler : Handler
    {
        protected readonly AngleCalculator angleCalculator;
        protected readonly CalibrationTarget leftTarget;
        protected readonly CalibrationTarget rightTarget;

        internal StorableFloat baseJsf;
        internal StorableFloat upJsf;
        internal StorableFloat downJsf;
        internal StorableFloat depthInJsf;
        internal StorableFloat depthOutJsf;
        internal StorableFloat sidewaysInJsf;
        internal StorableFloat sidewaysOutJsf;

        protected float UserUpMultiplier() => Calc.QuadraticRegression(baseJsf.val * upJsf.val);
        protected float UserDownMultiplier() => Calc.QuadraticRegression(baseJsf.val * downJsf.val);
        protected float UserDepthOutMultiplier() => Calc.QuadraticRegression(baseJsf.val * depthOutJsf.val);
        protected float UserDepthInMultiplier() => Calc.QuadraticRegression(baseJsf.val * depthInJsf.val);
        protected float UserSidewaysInMultiplier() => Calc.QuadraticRegression(baseJsf.val * sidewaysInJsf.val);
        protected float UserSidewaysOutMultiplier() => Calc.QuadraticRegression(baseJsf.val * sidewaysOutJsf.val);

        protected ForceHandler(ScriptBase script) : base(script)
        {
            angleCalculator = Script.angleCalculator;
            leftTarget = Script.leftCalibrationTarget;
            rightTarget = Script.rightCalibrationTarget;
        }

        protected StorableFloat NewMultiplierJsf(string name) => new StorableFloat(name, 1.00f, 0.00f, 2.00f)
        {
            setJSONCallbackFunction = OnMultiplierValueChanged,
        };

        protected abstract void OnMultiplierValueChanged(JSONStorableFloat jsf);

        internal override void SaveCalibrated()
        {
            baseJsf?.SaveCalibrated();
            upJsf?.SaveCalibrated();
            downJsf?.SaveCalibrated();
            depthInJsf?.SaveCalibrated();
            depthOutJsf?.SaveCalibrated();
            sidewaysInJsf?.SaveCalibrated();
            sidewaysOutJsf?.SaveCalibrated();
        }
    }
}
