using SimpleJSON;

namespace NAT.Common.Handlers
{
    sealed class PluginPresetHandler : PresetHandler
    {
        internal PluginPresetHandler(ScriptBase script, string subDir) : base(script, subDir)
        {
            var loadPresetWithPathAction = NewLoadWithPathAction("LoadPresetWithPath");
            loadPresetWithPathAction.actionCallback = LoadByTriggerAction;
            script.PreRegisterGeneralParam(loadPresetWithPathAction);
        }

        protected override void LoadAction(string path)
        {
            var json = script.LoadJSON(path);
            if(json != null)
            {
                var jc = json.AsObject;
                if(ValidatePreset(jc, path))
                {
                    script.RestoreFromPresetJSON(jc, path);
                }
            }
        }

        void LoadByTriggerAction(string path)
        {
            if(string.IsNullOrEmpty(path))
            {
                return;
            }

            OnPathSelected(ref path);
            LoadAction(path);
            loadPresetJsu.val = ""; // ensure empty value when next browse cancelled
        }

        bool ValidatePreset(JSONClass jc, string path)
        {
            string id = jc["id"].Value;
            if(!id.EndsWith($"Naturalis.{script.DomainName}"))
            {
                Loggr.Error($"Loaded preset is not a valid {script.DomainName} preset. Path: {path}", false);
                return false;
            }

            return true;
        }

        protected override void SaveAction(string path)
        {
            script.SavePresetJSON(script.GetJSON(), path);
            SuperController.singleton.DoSaveScreenshot(path);
        }

        internal void SaveDefaultPreset(bool isFemale)
        {
            FileUtils.EnsureDirExists(presetDir);
            string path = $"{presetDir}/Default{(isFemale ? "" : " (male)")}.json";
            script.SavePresetJSON(script.GetJSON(), path);
        }

        internal void LoadDefaultPreset(bool isFemale, bool autoLoading = false)
        {
            FileUtils.EnsureDirExists(presetDir);
            string path = $"{presetDir}/Default{(isFemale ? "" : " (male)")}.json";

            if(autoLoading)
            {
                if(FileUtils.FileExists(path))
                {
                    var json = script.LoadJSON(path);
                    if(json != null)
                    {
                        var jc = json.AsObject;
                        if(ValidatePreset(jc, path))
                        {
                            script.InitializeFromPresetJSON(jc, path);
                        }
                    }
                }
            }
            else
            {
                if(FileUtils.FileExists(path))
                {
                    LoadAction(path);
                }
                else
                {
                    script.Loggr.LogMessage("Default preset not saved yet.");
                }
            }
        }
    }
}
