using SimpleJSON;
using System.Text;

namespace NAT.Common.Handlers
{
    sealed class Migrater
    {
        readonly string _domainName;
        readonly SemVer _restoredFromVersion;
        readonly string _prereleaseTag;
        readonly JSONClass _jc;
        readonly StringBuilder _sb = new StringBuilder();

        internal Migrater(string domainName, SemVer version, JSONClass jc)
        {
            _domainName = domainName;
            _restoredFromVersion = version;
            _prereleaseTag = _restoredFromVersion?.prereleaseTag ?? "";
            _jc = jc;
        }

        internal string Migrate()
        {
            if(_restoredFromVersion == null)
            {
                return null;
            }

            _sb.Clear();
            MigrateTo1_1();
            MigrateTo1_2();
            return _sb.ToString();
        }

        void MigrateTo1_1()
        {
            if(_domainName == Constant.BOOTY_MAGIC)
            {
                MigrateToBootyMagic1_1();
            }
            else if(_domainName == Constant.TITTY_MAGIC)
            {
                MigrateToTittyMagic1_1();
            }
        }

        void MigrateToBootyMagic1_1()
        {
            if(_restoredFromVersion.IsEmpty || _restoredFromVersion.IsMajorMinorLt("1.1"))
            {
                Rename("Spring Offset", "Rotation Spring Offset");
                Rename("Damper Offset", "Rotation Damper Offset");
                Rename("Spring Back Gravity", "Rotation Spring Back Gravity");
                Rename("Spring Forward Gravity", "Rotation Spring Forward Gravity");
            }
        }

        void MigrateToTittyMagic1_1()
        {
            if(_restoredFromVersion.IsEmpty || _restoredFromVersion.IsMajorMinorLt("1.1"))
            {
                Rename("Force Physics Depth Out", "Force Physics Depth Out Multiplier");
                Rename("Force Physics Depth In", "Force Physics Depth In Multiplier");
                Rename("Spring Offset", "Rotation Spring Offset");
                Rename("Damper Offset", "Rotation Damper Offset");
                Rename("Spring Back Gravity", "Rotation Spring Back Gravity");
                Rename("Spring Forward Gravity", "Rotation Spring Forward Gravity");
                Rename("Damper Back Gravity", "Rotation Damper Back Gravity");
                Rename("Damper Forward Gravity", "Rotation Damper Forward Gravity");
                Rename("Damper Sideways In Gravity", "Rotation Damper Sideways In Gravity");
            }
        }

        void MigrateTo1_2()
        {
            if(_domainName == Constant.BOOTY_MAGIC)
            {
                MigrateToBootyMagic1_2();
            }
            else if(_domainName == Constant.TITTY_MAGIC)
            {
                MigrateToTittyMagic1_2();
            }
        }

        void MigrateToBootyMagic1_2()
        {
            if(_restoredFromVersion.IsEmpty || _restoredFromVersion.IsMajorMinorLt("1.2"))
            {
                _jc.Remove("Hard Colliders Base Mass Multiplier");
                _jc.Remove("Glute1 Mass Multiplier");
                _jc.Remove("Glute1 Radius Offset");
                _jc.Remove("Glute1 (1) Mass Multiplier");
                _jc.Remove("Glute1 (1) Radius Offset");
                _jc.Remove("Glute1 (2) Mass Multiplier");
                _jc.Remove("Glute1 (2) Radius Offset");
                _jc.Remove("Glute1 (3) Mass Multiplier");
                _jc.Remove("Glute1 (3) Radius Offset");
                _jc.Remove("Glute1 (4) Mass Multiplier");
                _jc.Remove("Glute1 (4) Radius Offset");
                _jc.Remove("Glute1 (5) Mass Multiplier");
                _jc.Remove("Glute1 (5) Radius Offset");
                _jc.Remove("Glute1 (6) Mass Multiplier");
                _jc.Remove("Glute1 (6) Radius Offset");
                _jc.Remove("Glute1 (7) Mass Multiplier");
                _jc.Remove("Glute1 (7) Radius Offset");
            }
            else if(_restoredFromVersion.IsEqual("1.2") && Utils.NewRegex("^beta[12]$").IsMatch(_prereleaseTag))
            {
                _jc.Remove("Hard Colliders Base Mass Multiplier");
                _jc.Remove("Glute1 Mass Multiplier");
                _jc.Remove("Glute1 (1) Mass Multiplier");
                _jc.Remove("Glute1 (2) Mass Multiplier");
                _jc.Remove("Glute1 (3) Mass Multiplier");
                _jc.Remove("Glute1 (4) Mass Multiplier");
                _jc.Remove("Glute1 (5) Mass Multiplier");
                _jc.Remove("Glute1 (6) Mass Multiplier");
                _jc.Remove("Glute1 (7) Mass Multiplier");
            }
        }

        void MigrateToTittyMagic1_2()
        {
            if(_restoredFromVersion.IsEmpty || _restoredFromVersion.IsMajorMinorLt("1.2"))
            {
                _jc.Remove("Hard Colliders Base Mass Multiplier");
                _jc.Remove("Pectoral1 Mass Multiplier");
                _jc.Remove("Pectoral1 Radius Offset");
                _jc.Remove("Pectoral2 Mass Multiplier");
                _jc.Remove("Pectoral2 Radius Offset");
                _jc.Remove("Pectoral3 Mass Multiplier");
                _jc.Remove("Pectoral3 Radius Offset");
                _jc.Remove("Pectoral4 Mass Multiplier");
                _jc.Remove("Pectoral4 Radius Offset");
                _jc.Remove("Pectoral5 Mass Multiplier");
                _jc.Remove("Pectoral5 Radius Offset");
                _jc["Up/Down Base Angle Offset"].AsFloat = -9.00f;
                _sb.Append(" Up/Down Base Angle reverted to zero (affects breast base shape).")
                    .Append(" You may want to adjust the base angle manually in the Physics tab's Up/Down Angle window.");

            }
            else if(_restoredFromVersion.IsEqual("1.2") && Utils.NewRegex("^beta[123]$").IsMatch(_prereleaseTag))
            {
                _jc.Remove("Hard Colliders Base Mass Multiplier");
                _jc.Remove("Pectoral1 Mass Multiplier");
                _jc.Remove("Pectoral2 Mass Multiplier");
                _jc.Remove("Pectoral3 Mass Multiplier");
                _jc.Remove("Pectoral4 Mass Multiplier");
                _jc.Remove("Pectoral5 Mass Multiplier");
            }

            bool is12Beta123 = _restoredFromVersion.IsEqual("1.2") && Utils.NewRegex("^beta[123]$").IsMatch(_prereleaseTag);
            if(_restoredFromVersion.IsEmpty || _restoredFromVersion.IsMajorMinorLt("1.2") || is12Beta123)
            {
                _jc.Remove("Up Gravity");
                _jc.Remove("Down Gravity");
                _jc.Remove("Sideways Gravity");
                _jc.Remove("Rotation Spring Offset");
                _jc.Remove("Up Angle Limit Offset");
                _jc.Remove("Down Angle Limit Offset");
                _jc.Remove("Up/Down Angle Target Offset");
                _jc.Remove("Up/Down Angle Target Up Gravity");
                _jc.Remove("Up/Down Angle Target Down Gravity");
                _jc.Remove("Right/Left Angle Limit Offset");
                _jc.Remove("Right/Left Angle Target Offset");
                _jc.Remove("Right/Left Angle Target Sideways In Gravity");
                _jc.Remove("Right/Left Angle Target Sideways Out Gravity");

                // technically required but affect only low softness breasts:
                // rotation damper
                // hard collider spring and damper
            }
            else if(_restoredFromVersion.IsEqual("1.2") && _prereleaseTag == "beta4")
            {
                _jc.Remove("Rotation Spring Offset");
                _jc.Remove("Down Gravity");
                _jc.Remove("Down Angle Limit Offset");
            }
        }

        void Rename(string fromKey, string toKey) => JSONUtils.RenameKey(_jc, fromKey, toKey);
    }
}
