using System.Collections.Generic;
using UnityEngine;

namespace NAT.Common.Models
{
    sealed class BodyPart
    {
        internal readonly ConfigurableJoint joint;
        internal readonly Rigidbody rigidbody;
        internal readonly DAZBone bone;
        internal readonly Rigidbody nippleRigidbody;
        internal readonly List<Rigidbody> colliderRigidbodies;

        internal BodyPart(ConfigurableJoint joint, Rigidbody nippleRigidbody = null)
        {
            this.joint = joint;
            rigidbody = joint.GetComponent<Rigidbody>();
            bone = joint.GetComponent<DAZBone>();
            this.nippleRigidbody = nippleRigidbody;
            colliderRigidbodies = new List<Rigidbody>(GetColliderRigidbodies());
        }

        IEnumerable<Rigidbody> GetColliderRigidbodies()
        {
            foreach(var rb in rigidbody.GetComponentsInChildren<Rigidbody>())
            {
                if(rb.name.EndsWith("NippleTrigger") || !rb.name.Contains("AutoCollider"))
                {
                    continue;
                }

                yield return rb;
            }
        }
    }
}
