using NAT.Common.Handlers.Configs;
using System.Text;
using UnityEngine;

namespace NAT.Common.Models
{
    sealed class ClothingItem : SimItem
    {
        internal readonly DAZClothingItem dazItem;
        internal readonly bool isBuiltIn;
        internal readonly string label;
        internal string ProfilesDir { get; private set; }
        internal int ProfilesDirHash { get; private set; }
        internal DAZClothingItemControl DazItemControl { get; private set; }
        internal ClothSimControl SimControl { get; private set; }
        internal DAZSkinWrap wrap;
        internal ClothingAdjustmentProfile chestProfile;
        internal ClothingAdjustmentProfile hipProfile;
        bool _isSim;

        internal ClothingItem(DAZClothingItem dazItem, string label, JSONStorableBool enabledJsb) : base(enabledJsb)
        {
            this.dazItem = dazItem;
            isBuiltIn = dazItem.creatorName == "MeshedVR";
            this.label = label;
        }

        internal void SetPresetSubDir(StringBuilder sb)
        {
            ProfilesDir = sb.Clear().AppendFormat("{0}/{1}", dazItem.creatorName.Trim(), dazItem.displayName.Trim()).ToString();
            ProfilesDirHash = Utils.Sha256HashToInt(ProfilesDir);
        }

        internal void RefreshControls()
        {
            if(!enabledJsb.val || retryCount > 10)
            {
                retryCount = 0;
                needsRetry = false;
                return;
            }

            if(needsRetry)
            {
                retryCount++;
            }

            needsRetry = !ConfirmControls();
            if(!needsRetry)
            {
                retryCount = 0;
                if(SimControl)
                {
                    simForceV3 = SimControl.GetVector3JSONParam("force");
                    _isSim = simForceV3 != null;
                }

                if(shouldSetSimForce)
                {
                    SetSimForce(simForce);
                }
            }
        }

        bool ConfirmControls()
        {
            if(!DazItemControl)
            {
                DazItemControl = dazItem.GetComponentInChildren<DAZClothingItemControl>();
            }

            if(!SimControl)
            {
                SimControl = dazItem.GetComponentInChildren<ClothSimControl>();
            }

            return DazItemControl && SimControl;
        }

        bool IsSimReady => _isSim && SimControl && SimControl.simEnabledJSON.val;

        internal void SetSimForce(Vector3 force)
        {
            if(IsSimReady)
            {
                simForceV3.val = force;
                SetGravityMultiplier(0);
            }
            else
            {
                simForce = force;
                shouldSetSimForce = true;
            }
        }

        internal void UnsetSimForce()
        {
            shouldSetSimForce = false;
            if(IsSimReady)
            {
                simForceV3.val = Vector3.zero;
                SetGravityMultiplier(SimControl.GetFloatParamValue("gravityMultiplier"));
            }
        }

        void SetGravityMultiplier(float multiplier)
        {
            if(SimControl.clothSettings)
            {
                SimControl.clothSettings.GravityMultiplier = multiplier;
            }
        }
    }
}
