using System.Text.RegularExpressions;

namespace NAT.Common.Models
{
    sealed class ConfigurableMorph : MorphWrapper
    {
        readonly JSONStorableFloat _softMultiplierJsf; // TODO refactor as a base multiplier
        readonly JSONStorableFloat _massMultiplierJsf;
        bool _isNegative;
        float _softMultiplier;
        float _massMultiplier;

        readonly string _label;
        internal readonly string morphDisplayName;

        internal ConfigurableMorph(DAZMorph morph, bool isNegative, JSONStorableFloat softMultiplierJsf, JSONStorableFloat massMultiplierJsf) : base(morph)
        {
            string morphName = morph.morphName;
            _label = IsCenterMorph() || IsLeftRightMorph()
                ? morph.displayName
                : Regex.Replace(morph.displayName, " L$", "");
            morphDisplayName = _label.Substring(morphName.IndexOf(" ") + 1);

            var isNegativeJsb = new JSONStorableBool("isNegative", isNegative);
            _softMultiplierJsf = softMultiplierJsf;
            _massMultiplierJsf = massMultiplierJsf;

            isNegativeJsb.setCallbackFunction = val => _isNegative = val;
            softMultiplierJsf.setCallbackFunction = val => _softMultiplier = val;
            massMultiplierJsf.setCallbackFunction = val => _massMultiplier = val;

            _isNegative = isNegativeJsb.val;
            _softMultiplier = softMultiplierJsf.val;
            _massMultiplier = massMultiplierJsf.val;
        }

        internal float Calculate(float effect, float mass, float softness)
        {
            float value =
                softness * _softMultiplier * effect +
                mass * _massMultiplier * effect;

            bool inRange = _isNegative ? value < 0 : value > 0;
            return inRange ? Calc.RoundToDecimals(value) : 0;
        }

        internal void SetValue(float effect, float mass, float softness) => SetValue(Calculate(effect, mass, softness));

        // bool IsCenterMorph() => !_morph.displayName.EndsWith(" L") && !_morph.displayName.EndsWith(" R");
        // bool IsLeftRightMorph() => _morph.displayName.StartsWith("LT") || _morph.displayName.StartsWith("RT");
        static bool IsCenterMorph() => false;
        static bool IsLeftRightMorph() => false;

        /* TODO broken */
        internal string ToCodeString(string getMorphMethodName, string subDir = "")
        {
            string dollar = IsCenterMorph() || IsLeftRightMorph() ? "" : "$";
            string sideStr = IsCenterMorph() || IsLeftRightMorph() ? "" : " {side}";

            return $@"
            new ConfigurableMorph(
                Person.{getMorphMethodName}({dollar}""{subDir}{_label}{sideStr}""),
                {_isNegative.ToString().ToLower()},
                new JSONStorableFloat(""softMultiplier"", {_softMultiplierJsf.val:0.00}f, -3.00f, 3.00f),
                new JSONStorableFloat(""massMultiplier"", {_massMultiplierJsf.val:0.00}f, -3.00f, 3.00f)
            ),";
        }
    }
}
