using UnityEngine;

namespace NAT.Common.Models
{
    sealed class HairItem : SimItem
    {
        readonly DAZHairGroup _dazItem;
        HairSimControl _simControl;
        bool _isSim;

        internal HairItem(DAZHairGroup dazItem, JSONStorableBool enabledJsb) : base(enabledJsb)
        {
            _dazItem = dazItem;
        }

        internal void RefreshControls()
        {
            if(!enabledJsb.val || retryCount > 10)
            {
                retryCount = 0;
                needsRetry = false;
                return;
            }

            if(needsRetry)
            {
                retryCount++;
            }

            if(!_simControl)
            {
                _simControl = _dazItem.GetComponentInChildren<HairSimControl>();
            }

            needsRetry = !_simControl;
            if(!needsRetry)
            {
                retryCount = 0;
                if(_simControl)
                {
                    _isSim = true;
                    simForceV3 = _simControl.GetVector3JSONParam("wind");
                }

                if(shouldSetSimForce)
                {
                    SetSimForce(simForce);
                }
            }
        }

        internal void SetSimForce(Vector3 force)
        {
            if(!_isSim)
            {
                return;
            }

            if(!_simControl || !_simControl.GetBoolParamValue("simulationEnabled"))
            {
                simForce = force;
                shouldSetSimForce = true;
                return;
            }

            shouldSetSimForce = false;

            // Sanity check
            if(simForceV3 == null)
            {
                Loggr.Error($"{_dazItem.uid}: wind storable was null!");
                return;
            }

            if(_simControl.hairSettings)
            {
                _simControl.hairSettings.PhysicsSettings.GravityMultiplier = 0f;
            }

            simForceV3.val = force;
        }

        internal void UnsetSimForce()
        {
            shouldSetSimForce = false;

            if(!_simControl || !_simControl.GetBoolParamValue("simulationEnabled"))
            {
                return;
            }

            // Sanity check
            if(simForceV3 == null)
            {
                Loggr.Error($"{_dazItem.uid}: force storable was null!");
                return;
            }

            if(_simControl.hairSettings)
            {
                _simControl.hairSettings.PhysicsSettings.GravityMultiplier = _simControl.GetFloatParamValue("gravityMultiplier");
            }

            simForceV3.val = Vector3.zero;
        }
    }
}
