﻿using System.Collections.Generic;
using UnityEngine;

namespace NAT.Common.Models
{
    sealed class HardColliderPair
    {
        internal readonly string id;
        readonly float _autoRadiusMultiplier;

        internal readonly HardCollider left;
        internal readonly HardCollider right;

        internal HardColliderPair(
            string id,
            HardCollider left,
            HardCollider right,
            float autoRadiusMultiplier
        )
        {
            this.id = id;
            this.left = left;
            this.right = right;
            _autoRadiusMultiplier = autoRadiusMultiplier;
        }

        internal void UpdateFriction(float maxFriction, IEnumerable<Rigidbody> rigidbodiesLeft, IEnumerable<Rigidbody> rigidbodiesRight)
        {
            left.UpdateFriction(maxFriction, rigidbodiesLeft);
            right.UpdateFriction(maxFriction, rigidbodiesRight);
        }

        internal void UpdateFriction(float maxFriction, Vector3[] skinVerts, IEnumerable<int> protrusionIndicesLeft, IEnumerable<int> protrusionIndicesRight)
        {
            left.UpdateFriction(maxFriction, skinVerts, protrusionIndicesLeft);
            right.UpdateFriction(maxFriction, skinVerts, protrusionIndicesRight);
        }

        internal void SetClothingMassMultipliers(float massMultiplierLeft, float massMultiplierRight)
        {
            right.SetClothingMassMultiplier(massMultiplierRight);
            left.SetClothingMassMultiplier(massMultiplierLeft);
        }

        internal void CalculateBaseRadius(Vector3[] skinVerts)
        {
            left.CalculateBaseRadius(skinVerts, _autoRadiusMultiplier);
            right.CalculateBaseRadius(skinVerts, _autoRadiusMultiplier);
        }

        internal void PreRegisterParams(ScriptBase script)
        {
            script.PreRegisterHardCollidersParam(left.massMultiplierJsf);
            script.PreRegisterHardCollidersParam(left.springMultiplierJsf);
            script.PreRegisterHardCollidersParam(left.damperMultiplierJsf);
            script.PreRegisterHardCollidersParam(left.radiusOffsetJsf);
            script.PreRegisterHardCollidersParam(left.depthJsf);
        }

        internal void ConfigureAutoColliders()
        {
            left.ConfigureAutoColliders();
            right.ConfigureAutoColliders();
        }

        internal void RestoreOriginalValues()
        {
            left.RestoreOriginalValues();
            right.RestoreOriginalValues();
        }

        internal bool HasRigidbodies() => left.autoCollider.jointRB && right.autoCollider.jointRB;
    }
}
