using NAT.Common.Handlers.Configs;
using NAT.Common.Storables;
using UnityEngine;

namespace NAT.Common.Models
{
    abstract class PhysicsParam
    {
        internal readonly string paramName;
        protected readonly string paramId;

        internal readonly JSONStorableFloat baseValueJsf;
        internal readonly StorableFloat offsetJsf;

        readonly PhysicsCalculator _calculator = new PhysicsCalculator();
        readonly PhysicsCalculator _quicknessOffsetCalculator = new PhysicsCalculator();
        readonly PhysicsCalculator _slownessOffsetCalculator = new PhysicsCalculator();

        internal MultiplierConfig PrimaryConfig
        {
            set { _calculator.primaryConfig = value; }
        }

        internal MultiplierConfig SecondaryConfig
        {
            set { _calculator.secondaryConfig = value; }
        }

        internal MultiplierConfig QuicknessConfig
        {
            set { _quicknessOffsetCalculator.primaryConfig = value; }
        }

        internal MultiplierConfig SlownessConfig
        {
            set { _slownessOffsetCalculator.primaryConfig = value; }
        }

        protected PhysicsParam(string side, string name, float min, float max)
        {
            paramName = name;
            paramId = Utils.ParamName(name, side);
            baseValueJsf = new StorableFloat($"{paramId} Base Value", 0, min, max);
            // arbitrarily high min/max to allow restoring any value from JSON; clamped later with UpdateOffsetMinMax()
            offsetJsf = new StorableFloat($"{paramId} Offset", 0, -10000, 10000);
        }

        internal abstract void DoSync();

        internal void UpdateOffsetMinMax()
        {
            offsetJsf.min = -(baseValueJsf.val - baseValueJsf.min);
            offsetJsf.max = baseValueJsf.max - baseValueJsf.val;
        }

        internal void UpdateBaseValue(float mass, float softness, float quickness, float physicsRateMultiplier, bool usePrimary)
        {
            float value = _calculator.Calculate(mass, softness, usePrimary);

            if(quickness > 0)
            {
                float maxQuicknessOffset = _quicknessOffsetCalculator.Calculate(mass, softness, true);
                value += Mathf.Lerp(0, maxQuicknessOffset, quickness);
            }

            if(quickness < 0)
            {
                float maxSlownessOffset = _slownessOffsetCalculator.Calculate(mass, softness, true);
                value += Mathf.Lerp(0, maxSlownessOffset, -quickness);
            }

            baseValueJsf.val = physicsRateMultiplier * value;
            DoSync();
        }

        internal void UpdateOffsetValue(float value)
        {
            offsetJsf.valNoCallback = value;
            DoSync();
        }
    }
}
