using System;

namespace NAT.Common.Models
{
    abstract class PhysicsParamPair
    {
        protected readonly ScriptBase script;
        internal readonly string name;
        internal readonly string buttonLabel;
        internal string infoText;
        internal bool usesRealMass;
        internal bool dependsOnPhysicsRate;

        internal float ValuePrecision { get; private set; }

        string _valueFormat;

        internal string ValueFormat
        {
            get { return _valueFormat; }
            set
            {
                _valueFormat = value;
                /* 0.5f is to ensure changes slightly smaller than the slider valueformat are not ignored,
                 * as they can get rounded to a value different from the current value
                 */
                ValuePrecision = 0.5f * ConvertFormatToPrecision(_valueFormat);
            }
        }

        static float ConvertFormatToPrecision(string valueFormat)
        {
            try
            {
                string value = valueFormat.Replace("F", "");
                int parsedValue = int.Parse(value);
                float precision = 1f / (float) Math.Pow(10, parsedValue);
                return precision;
            }
            catch(Exception e)
            {
                Loggr.Error($"Failed to parse valueFormat: {valueFormat}. Error: {e}");
                return 0.01f;
            }
        }

        protected PhysicsParam leftParam, rightParam;

        protected PhysicsParamPair(ScriptBase script, string name, string buttonLabel)
        {
            this.script = script;
            this.name = name;
            this.buttonLabel = buttonLabel ?? name;
        }

        internal void UpdateBaseValue(float mass, float softness, float quickness, bool usePrimary = true)
        {
            float physicsRateMultiplier = dependsOnPhysicsRate ? Utils.PhysicsRateMultiplier() : 1;
            leftParam.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
            rightParam.UpdateBaseValue(mass, softness, quickness, physicsRateMultiplier, usePrimary);
        }

        internal void UpdateOffsetMinMax()
        {
            leftParam.UpdateOffsetMinMax();
            rightParam.UpdateOffsetMinMax();
        }

        internal virtual bool IsModified => name != FAT_COLLIDER_FRICTION.NAME && leftParam.offsetJsf.val != 0;
    }
}
