using SimpleJSON;
using System;
using UnityEngine;

namespace NAT.Common.Models
{
    sealed class SoftJointCalibration
    {
        float _baseValueUp;
        float _baseValueDown;
        float _baseValueForward;
        float _baseValueBack;
        float _baseValueLeft;
        float _baseValueRight;

        internal float BaseValue { get; private set; }

        internal void SetBaseValue(string direction, float value)
        {
            switch(direction)
            {
                case Direction.UP:
                    _baseValueUp = value;
                    break;
                case Direction.DOWN:
                    _baseValueDown = value;
                    break;
                case Direction.FORWARD:
                    _baseValueForward = value;
                    break;
                case Direction.BACK:
                    _baseValueBack = value;
                    break;
                case Direction.LEFT:
                    _baseValueLeft = value;
                    break;
                case Direction.RIGHT:
                    _baseValueRight = value;
                    break;
                default: throw new ArgumentException($"Invalid direction {direction}");
            }
        }

        internal void CalculateInterpBaseValue(float roll, float pitch)
        {
            float distortionBasePitch;
            if(pitch > 0) // leaning forward
            {
                distortionBasePitch = pitch < 1
                    ? Mathf.Lerp(_baseValueDown, _baseValueForward, pitch) // upright
                    : Mathf.Lerp(_baseValueForward, _baseValueUp, pitch - 1); // upside down
            }
            else // leaning back
            {
                distortionBasePitch = pitch > -1
                    ? Mathf.Lerp(_baseValueDown, _baseValueBack, -pitch) // upright
                    : Mathf.Lerp(_baseValueBack, _baseValueUp, -pitch - 1); // upside down
            }

            BaseValue = roll > 0
                ? Mathf.Lerp(distortionBasePitch, _baseValueLeft, roll) // leaning left
                : Mathf.Lerp(distortionBasePitch, _baseValueRight, -roll); // leaning right
        }

        internal JSONClass SerializeCalibrationData() => new JSONClass
        {
            ["up"] = { AsFloat = _baseValueUp },
            ["down"] = { AsFloat = _baseValueDown },
            ["forward"] = { AsFloat = _baseValueForward },
            ["back"] = { AsFloat = _baseValueBack },
            ["left"] = { AsFloat = _baseValueLeft },
            ["right"] = { AsFloat = _baseValueRight },
        };

        internal void CalibrateFromJSON(JSONClass jc)
        {
            _baseValueUp = jc["up"].AsFloat;
            _baseValueDown = jc["down"].AsFloat;
            _baseValueForward = jc["forward"].AsFloat;
            _baseValueBack = jc["back"].AsFloat;
            _baseValueLeft = jc["left"].AsFloat;
            _baseValueRight = jc["right"].AsFloat;
        }
    }
}
