using NAT.Common.Handlers.Configs;
using NAT.Common.Storables;
using System;

namespace NAT.Common.Models
{
    sealed class SoftPhysicsParam : PhysicsParam
    {
        internal PhysicsCalculator inverseFrictionCalculator;
        float _additiveFrictionAdjustment;

        internal readonly GradientConfig gradientConfig;
        internal readonly SoftJointDistortionConfig distortionConfig;

        readonly Action<SoftPhysicsParam, float> _sync;

        internal SoftPhysicsParam(string domainName, string side, string name, float min, float max, Action<SoftPhysicsParam, float> sync)
            : base(side, name, min, max)
        {
            _sync = sync;

            gradientConfig = new GradientConfig
            {
                middleShiftJsf = new StorableFloat($"{paramId} Gradient Middle Shift", 0.00f, -1.00f, 1.00f),
                outerMultiplierJsf = NewGroupMultiplierJsf(paramId, "Outer"),
                centerMultiplierJsf = NewGroupMultiplierJsf(paramId, "Center"),
            };

            if(domainName == Constant.TITTY_MAGIC)
            {
                gradientConfig.nippleMultiplierJsf = NewGroupMultiplierJsf(paramId, "Nipple");
            }

            distortionConfig = new SoftJointDistortionConfig
            {
                compressionEffectJsf = new StorableFloat($"{paramId} Local Compression Effect", 0, -3.00f, 3.00f),
                compressionSensitivityJsf = new StorableFloat($"{paramId} Local Compression Sensitivity", 0, 0.00f, 1.00f),
                stretchingEffectJsf = new StorableFloat($"{paramId} Local Stretching Effect", 0, -3.00f, 3.00f),
                stretchingSensitivityJsf = new StorableFloat($"{paramId} Local Stretching Sensitivity", 0, 0.00f, 1.00f),
            };
        }

        static StorableFloat NewGroupMultiplierJsf(string paramName, string groupName) => new StorableFloat(
            $"{paramName} Gradient {groupName} Multiplier",
            1.00f,
            0.01f,
            Constant.GRADIENT_MULTI_MAX
        );

        internal override void DoSync() =>
            _sync(this, _additiveFrictionAdjustment);

        internal void UpdateInverseFrictionValue(float friction, float mass, float softness)
        {
            float value = inverseFrictionCalculator.CalculateDynamic(friction, mass, softness, true);
            _additiveFrictionAdjustment = Calc.RoundToDecimals(value, 10000);
            DoSync();
        }

        internal void ResetInverseFrictionValue()
        {
            _additiveFrictionAdjustment = 0;
            DoSync();
        }
    }
}
