using SimpleJSON;

namespace NAT.Common.Storables
{
    sealed class StorableBool : JSONStorableBool, IStorableParam
    {
        bool _calibratedValue;
        public string Name => name;
        public bool Temporary { get; set; }
        public bool IsModified => val != defaultVal;
        public bool IsCalibrated => val == _calibratedValue;

        internal StorableBool(string paramName, bool startingValue) : base(paramName, startingValue)
        {
            storeType = StoreType.Full;
        }

        internal void SaveCalibrated()
        {
            _calibratedValue = val;
        }

        /* Implement splitting by ':' */
        public override bool StoreJSON(
            JSONClass jc,
            bool includePhysical = true,
            bool includeAppearance = true,
            bool forceStore = false
        )
        {
            bool needsStore = NeedsStore(jc, includePhysical, includeAppearance) && (forceStore || _val != _defaultVal);
            if(needsStore)
            {
                if(name.IndexOf(':') != -1)
                {
                    string[] strArray = name.Split(':');
                    jc[strArray[0]][strArray[1]].AsBool = _val;
                }
                else
                {
                    jc[name].AsBool = _val;
                }
            }

            return needsStore;
        }

        /* Implement splitting by ':' */
        public override void RestoreFromJSON(
            JSONClass jc,
            bool restorePhysical = true,
            bool restoreAppearance = true,
            bool setMissingToDefault = true
        )
        {
            if(!NeedsRestore(jc, restorePhysical, restoreAppearance))
            {
                return;
            }

            if(name.IndexOf(':') != -1)
            {
                ParseNestedValue(name.Split(':'), jc, setMissingToDefault);
            }
            else if(jc[name] != null)
            {
                val = jc[name].Value == string.Empty ? _defaultVal : jc[name].AsBool;
            }
            else if(altName != null)
            {
                if(altName.IndexOf(':') != -1)
                {
                    ParseNestedValue(altName.Split(':'), jc, setMissingToDefault);
                }

                if(jc[altName] != null)
                {
                    val = jc[altName].AsBool;
                }
                else
                {
                    if(!setMissingToDefault)
                    {
                        return;
                    }

                    val = _defaultVal;
                }
            }
            else
            {
                if(!setMissingToDefault)
                {
                    return;
                }

                val = _defaultVal;
            }
        }

        void ParseNestedValue(string[] strArray, JSONClass jc, bool setMissingToDefault)
        {
            if(jc[strArray[0]][strArray[1]] != null)
            {
                val = jc[strArray[0]][strArray[1]].AsBool;
            }
            else
            {
                if(!setMissingToDefault)
                {
                    return;
                }

                val = _defaultVal;
            }
        }

        public void RegisterTo(MVRScript script) => script.RegisterBool(this);
    }
}
