namespace NAT.Common.Storables
{
    sealed class StorableFloat : JSONStorableFloat, IStorableParam
    {
        const float TOLERANCE = 0.00001f;
        float _calibratedValue;

        public string Name => name;
        public bool Temporary { get; set; }
        public bool IsModified => !val.IsEqual(_defaultVal, TOLERANCE);
        public bool IsCalibrated => val.IsEqual(_calibratedValue, TOLERANCE);

        internal StorableFloat(
            string paramName,
            float startingValue,
            float minValue,
            float maxValue,
            bool constrain = true,
            bool interactable = true
        ) : base(paramName, startingValue, minValue, maxValue, constrain, interactable)
        {
            storeType = StoreType.Full;
        }

        internal void SetValueAndDefault(float value)
        {
            defaultVal = value;
            val = value;
        }

        // ReSharper disable once UnusedMember.Global
        internal void Callback() => setCallbackFunction?.Invoke(val);
        internal void SaveCalibrated() => _calibratedValue = val;
        public void RegisterTo(MVRScript script) => script.RegisterFloat(this);

        internal static StorableFloat Copy(JSONStorableFloat source, string prefix = "", string suffix = "", bool temporary = false)
        {
            var copy = new StorableFloat($"{prefix}{source.name}{suffix}", source.defaultVal, source.min, source.max);
            if(temporary)
            {
                copy.Temporary = true;
            }

            return copy;
        }
    }
}
