using System;
using UnityEngine;

namespace NAT.Common.UI
{
    class Element
    {
        readonly UIDynamic _uiDynamic;
        bool _prevActive = true;

        internal Element(UIDynamic uiDynamic)
        {
            _uiDynamic = uiDynamic;
        }

        internal bool IsVisible => _uiDynamic.gameObject.activeSelf;
        internal bool isAnimating;
        internal void Show() => _uiDynamic.gameObject.SetActive(true);
        internal void ShowPrev() => _uiDynamic.gameObject.SetActive(_prevActive);

        internal void Hide()
        {
            bool isActive = _uiDynamic.gameObject.activeSelf;
            _uiDynamic.gameObject.SetActive(false);
            ClosePopup();
            _prevActive = isActive;
        }

        internal void SetActiveStyle(bool isActive, bool setInteractable = false, bool highlightIneffective = false) =>
            _uiDynamic.SetActiveStyle(isActive, setInteractable, highlightIneffective);

        internal void SetLabel(string label) => _uiDynamic.SetLabel(label);

        internal void ClosePopup() => (_uiDynamic as UIDynamicPopup)?.SetVisible(false);

        internal T GetUIDynamicAs<T>()
        {
            if(_uiDynamic is T)
            {
                return (T) Convert.ChangeType(_uiDynamic, typeof(T));
            }

            return default(T);
        }

        internal RectTransform Rect => _uiDynamic.transform.GetComponent<RectTransform>();
    }
}
