using NAT.Common.Models;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace NAT.Common.UI
{
    sealed class ParamValueInfo
    {
        readonly JointPhysicsParamPair _paramPair;
        readonly JointPhysicsParam _param;
        readonly string _label;
        readonly string _units;
        internal UIDynamicTextField textField;
        internal UIDynamicTextField gravityTargetsField;

        float _prevValue = float.NaN;
        readonly Dictionary<string, float> _prevGravityTargets;

        internal ParamValueInfo(JointPhysicsParamPair paramPair, string label = "Value", string units = null)
        {
            _paramPair = paramPair;
            _param = paramPair.Left;
            if(_param.HasAdjustableGravityPhysics())
            {
                _prevGravityTargets = _param.gravityCalculators.ToDictionary(
                    pair => pair.Key,
                    pair => float.NaN
                );
            }

            _label = label;
            _units = units;
        }

        internal void Update(StringBuilder sb)
        {
            float value = _paramPair.Left.Value;
            if(value.IsEqual(_prevValue, _paramPair.ValuePrecision))
            {
                return;
            }

            _prevValue = value;
            sb.Clear();
            if(string.IsNullOrEmpty(_units))
            {
                sb.AppendFormat("{0}: <b>{1}</b>", _label, value.ToString(_paramPair.ValueFormat));
            }
            else
            {
                sb.AppendFormat("{0}: <b>{1} {2}</b>", _label, value.ToString(_paramPair.ValueFormat), _units);
            }

            textField.text = sb.ToString();
        }

        internal void UpdateGravityTargets(StringBuilder sb, float mass, float softness)
        {
            if(gravityTargetsField == null)
            {
                return;
            }

            sb.Clear();
            bool doUpdateText = false;
            foreach(var pair in _param.gravityCalculators)
            {
                var config = pair.Value;
                float targetValue =
                    _param.offsetJsf.val +
                    _param.baseValueJsf.val +
                    config.CalculateDynamic(1f, mass, softness, true);
                targetValue = Mathf.Clamp(targetValue, _param.baseValueJsf.min, _param.baseValueJsf.max);
                if(!targetValue.IsEqual(_prevGravityTargets[pair.Key], _paramPair.ValuePrecision))
                {
                    _prevGravityTargets[pair.Key] = targetValue;
                    doUpdateText = true;
                }

                sb.AppendFormat("\n{0}: <b>{1}</b>", pair.Key, targetValue.ToString(_paramPair.ValueFormat));
            }

            if(doUpdateText)
            {
                gravityTargetsField.text = sb.ToString();
            }
        }
    }
}
