using UnityEngine;
using UnityEngine.Events;

namespace NAT.Common.UI
{
    sealed class NavigationButton : Element
    {
        readonly UIDynamicButton _uiDynamicButton;
        bool _disabled;
        readonly Color _activeColor;

        internal NavigationButton(UIDynamicButton uiDynamicButton, Color activeColor, Color buttonColor) : base(uiDynamicButton)
        {
            _uiDynamicButton = uiDynamicButton;
            _uiDynamicButton.buttonColor = buttonColor;
            _activeColor = activeColor;
        }

        internal void SetActive()
        {
            if(_disabled)
            {
                return;
            }

            _uiDynamicButton.textColor = _activeColor;
            var colors = _uiDynamicButton.button.colors;
            colors.highlightedColor = Color.white;
            colors.pressedColor = Color.white;
            _uiDynamicButton.button.colors = colors;
        }

        internal void SetInactive()
        {
            if(_disabled)
            {
                return;
            }

            _uiDynamicButton.textColor = Color.white;
            var colors = _uiDynamicButton.button.colors;
            colors.highlightedColor = Colors.defaultBtnHighlightedColor;
            colors.pressedColor = Colors.defaultBtnPressedColor;
            _uiDynamicButton.button.colors = colors;
        }

        internal void SetEnabled(bool enabled)
        {
            if(enabled && !_disabled)
            {
                return;
            }

            _disabled = !enabled;
            if(_disabled)
            {
                _uiDynamicButton.textColor = new Color(0.8f, 0.8f, 0.8f);
                _uiDynamicButton.button.interactable = false;
            }
            else
            {
                SetInactive();
                _uiDynamicButton.button.interactable = true;
            }
        }

        internal void AddListener(UnityAction unityAction)
        {
            _uiDynamicButton.AddListener(unityAction);
        }
    }
}
