using System;
using System.Collections.Generic;

namespace NAT.Common.UI
{
    sealed class Section
    {
        readonly Dictionary<string, Element> _elements = new Dictionary<string, Element>();
        readonly Action<Section> _buildAction;
        internal NavigationButton navigationButton;
        internal bool IsShown { get; private set; }

        internal Section(Action<Section> buildAction)
        {
            _buildAction = buildAction;
        }

        internal void AddElement(string name, Element element) => _elements.Add(name, element);

        internal Element GetElement(string name)
        {
            Element element;
            _elements.TryGetValue(name, out element);
            return element;
        }

        internal void Show(bool showPrev = false)
        {
            navigationButton?.SetActive();
            if(_elements.Count > 0)
            {
                foreach(var pair in _elements)
                {
                    if(showPrev)
                    {
                        pair.Value.ShowPrev();
                    }
                    else
                    {
                        pair.Value.Show();
                    }
                }
            }
            else
            {
                _buildAction(this);
            }

            IsShown = true;
        }

        internal void Hide()
        {
            navigationButton?.SetInactive();
            foreach(var pair in _elements)
            {
                pair.Value.Hide();
            }

            IsShown = false;
        }

        public void SetActiveStyle(bool isActive, bool setInteractable = false)
        {
            foreach(var pair in _elements)
            {
                pair.Value.SetActiveStyle(isActive, setInteractable);
            }
        }
    }
}
