using NAT.Common.Handlers;
using System.Text;
using UnityEngine;

namespace NAT.Common.UI.Windows
{
    sealed class AdditionalClothingSettingsWindow : Window
    {
        readonly ClothingHandler _clothingHandler;
        UIDynamicTextField _profileTransitionStatusTextField;

        internal AdditionalClothingSettingsWindow(ScriptBase script) : base(script, nameof(AdditionalClothingSettingsWindow))
        {
            _clothingHandler = script.clothingHandler;
            showInfoPanel = true;
        }

        protected override void Build()
        {
            AddElement("Target Profile Transition Header", _ =>
            {
                string text = new StringBuilder()
                    .AppendSize("\n", 20)
                    .AppendBold("Target Profile Transition")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -210), new Vector2(-558, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            AddElement("Select Target Profile Preset", name =>
            {
                var uiDynamic = CreateButton(name, new Vector2(10, -275), new Vector2(-683, 50));
                uiDynamic.AddListener(() => _clothingHandler.LoadProfile(true));
                return uiDynamic;
            });

            AddElement("Set Profile Off As Target", name =>
            {
                var uiDynamic = CreateButton(name, new Vector2(405, -275), new Vector2(-730, 50));
                uiDynamic.AddListener(() => _clothingHandler.SetProfileOffAsTarget(_clothingHandler.SelectedProfile));
                return uiDynamic;
            });

            AddElement("Reset", name =>
            {
                var uiDynamic = CreateButton(name, new Vector2(753, -275), new Vector2(-950, 50));
                uiDynamic.AddListener(() => _clothingHandler.ResetTargetProfile());
                return uiDynamic;
            });

            AddElement("Profile Transition Info", _ =>
            {
                string text = new StringBuilder()
                    .Append("Transition between the current and target profiles.")
                    .AppendSize("\n", 8)
                    .AppendLine()
                    .Append("The selected target profile and the transition percentage save with the profile.")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(545, -530), new Vector2(-555, 160));
                uiDynamic.UItext.fontSize = 26;
                uiDynamic.UItext.alignment = TextAnchor.UpperLeft;
                return uiDynamic;
            });

            AddElement("Profile Transition Status", _ =>
            {
                var uiDynamic = CreateTextField("", new Vector2(10, -390), new Vector2(-20, 100));
                uiDynamic.UItext.fontSize = 26;
                uiDynamic.UItext.alignment = TextAnchor.UpperLeft;
                _profileTransitionStatusTextField = uiDynamic;
                return uiDynamic;
            });

            AddSlider(_clothingHandler.ProfileTransitionJsf, name => CreateSlider(name, "F0", new Vector2(10, -485)));

            AddElement("Set Current State As Current Profile", name =>
            {
                var uiDynamic = CreateButton(name, new Vector2(10, -545), new Vector2(-555, 50));
                uiDynamic.AddListener(() => _clothingHandler.SetCurrentTransitionStateAsCurrentProfile());
                return uiDynamic;
            });

            AddElement("Other", _ =>
            {
                string text = new StringBuilder()
                    .AppendSize("\n", 20)
                    .AppendBold("Other")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -630), new Vector2(-558, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            AddSlider(_clothingHandler.MorphingTransitionJsf, name => CreateSlider(name, "F2", new Vector2(10, -755)));

            AddElement("Morphing Transition Info", _ =>
            {
                string text = new StringBuilder()
                    .Append("Timer for how quickly morphing offsets are animated when a clothing item or its profile is toggled on/off.")
                    .AppendFormat(" This saves with the current {0} instance.", Constant.NATURALIS)
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(545, -755), new Vector2(-558, 120));
                uiDynamic.UItext.fontSize = 26;
                uiDynamic.UItext.alignment = TextAnchor.UpperLeft;
                return uiDynamic;
            });

            AddToggle(_clothingHandler.ProfileOffWhenUndressingJsb, name => CreateToggle(name, new Vector2(10, -845), -555));

            AddElement("Undress Info", _ =>
            {
                const string text = "Deactivates the profile when Undress is toggled on in the item's physics settings (if sim item). This saves with the profile.";
                var uiDynamic = CreateTextField(text, new Vector2(545, -915), new Vector2(-558, 120));
                uiDynamic.UItext.fontSize = 26;
                uiDynamic.UItext.alignment = TextAnchor.UpperLeft;
                return uiDynamic;
            });
        }

        protected override void PostShow() => Refresh();

        internal override void Refresh()
        {
            if(!isBuilt)
            {
                return;
            }

            var profileTransitionSlider = GetElement(_clothingHandler.ProfileTransitionJsf.name);
            profileTransitionSlider.SetActiveStyle(_clothingHandler.SelectedProfile != null, true);
        }

        internal override void UpdateDiagnostics(bool atInterval)
        {
            if(!isBuilt)
            {
                return;
            }

            if(atInterval)
            {
                // TODO on callback
                _profileTransitionStatusTextField.text = ClothingUIHelpers.TransitionStatus(infoSb, _clothingHandler);
            }
        }

        internal override string GetInfo() => ((ClothingWindow) parent).GetInfo();
    }
}
