using NAT.Common.Models;
using System.Collections.Generic;
using UnityEngine;

namespace NAT.Common.UI.Windows
{
    sealed class FatBackForceWindow : Window
    {
        readonly JointPhysicsParamPair _force;
        readonly JointPhysicsParamPair _maxForce;
        readonly JointPhysicsParamPair _threshold;
        readonly List<ParamValueInfo> _paramValueInfos = new List<ParamValueInfo>();

        internal FatBackForceWindow(
            ScriptBase script,
            JointPhysicsParamPair force,
            JointPhysicsParamPair maxForce,
            JointPhysicsParamPair threshold
        ) : base(script, force.name)
        {
            _force = force;
            _maxForce = maxForce;
            _threshold = threshold;
        }

        protected override void Build()
        {
            BuildParamArea(_force, 0);
            BuildParamArea(_maxForce, -220);
            BuildParamArea(_threshold, -440);
        }

        void BuildParamArea(JointPhysicsParamPair paramPair, int yOffset)
        {
            var paramValueInfo = new ParamValueInfo(paramPair);
            _paramValueInfos.Add(paramValueInfo);

            AddElement($"{paramPair.name} Info", _ =>
            {
                var uiDynamic = CreateTextField(paramPair.infoText, new Vector2(10, -430 + yOffset), new Vector2(-555, 280));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddSlider(paramPair.Left.offsetJsf, _ => CreateSlider("Offset", paramPair.ValueFormat, new Vector2(545, -265 + yOffset)));

            AddElement($"{paramPair.name} Value", _ =>
            {
                var uiDynamic = CreateTextField("", new Vector2(545, -330 + yOffset), new Vector2(-555, 45));
                uiDynamic.UItext.fontSize = 28;
                paramValueInfo.textField = uiDynamic;
                return uiDynamic;
            });
        }

        internal override void UpdateDiagnostics(bool atInterval)
        {
            if(!isBuilt || !atInterval)
            {
                return;
            }

            // TODO separate value indicators for different sources e.g. clothing
            foreach(var paramValueInfo in _paramValueInfos)
            {
                paramValueInfo.Update(infoSb);
            }
        }
    }
}
