using NAT.Common.Handlers;
using System.Linq;
using System.Text;
using UnityEngine;
using UnityEngine.UI;

namespace NAT.Common.UI.Windows
{
    sealed class MorphsWindow : Window
    {
        readonly ForceMorphHandler _forceMorphHandler;

        internal MorphsWindow(ScriptBase script) : base(script)
        {
            _forceMorphHandler = script.forceMorphHandler;
        }

        protected override void Build()
        {
            AddElement("Directional Force Morphing", name =>
            {
                var uiDynamic = CreateTextField(name.Bold(), new Vector2(10, -165), new Vector2(-555, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            if(script.IsBootyMagic)
            {
                BuildBootyMagicUI();
            }
            else if(script.IsTittyMagic)
            {
                BuildTittyMagicUI();
            }
        }

        void BuildBootyMagicUI()
        {
            AddElement("Info1", _ =>
            {
                const string text = "Adjust the amount of morphing and the joint's neutral position and rotation.";
                var uiDynamic = CreateTextField(text, new Vector2(10, -280), new Vector2(-555, 118));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddSlider(_forceMorphHandler.baseJsf, _ => CreateSlider("Base Multiplier", "F2", new Vector2(545, -225)));
            AddSlider(script.upDownAngleOffsetJsf, name => CreateSlider($"{name} (\u00b0)", "F1", new Vector2(10, -375)));

            AddMultiplierSlider(_forceMorphHandler.upJsf, "Up", new Vector2(545, -375));
            AddMultiplierSlider(_forceMorphHandler.downJsf, "Down", new Vector2(545, -505));

            AddElement("Info2", _ =>
            {
                const string text = "Up/Down morphing is based on the joint rotating away from the calibrated vertical angle (plus offset).";
                var uiDynamic = CreateTextField(text, new Vector2(10, -505), new Vector2(-555, 118));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddSlider(script.depthOffsetJsf, name => CreateSlider($"{name} (mm)", "F1", new Vector2(10, -655)));

            AddMultiplierSlider(_forceMorphHandler.depthOutJsf, "Depth Out", new Vector2(545, -655));
            AddMultiplierSlider(_forceMorphHandler.depthInJsf, "Depth In", new Vector2(545, -785));

            AddElement("Info3", _ =>
            {
                const string text = "Depth morphing is based on the joint's movement away from the calibrated Z axis position (plus offset).";
                var uiDynamic = CreateTextField(text, new Vector2(10, -785), new Vector2(-555, 118));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddSlider(script.leftRightAngleOffsetJsf, name => CreateSlider($"{name} (\u00b0)", "F1", new Vector2(10, -935)));

            AddMultiplierSlider(_forceMorphHandler.sidewaysInJsf, "Sideways In", new Vector2(545, -935));
            AddMultiplierSlider(_forceMorphHandler.sidewaysOutJsf, "Sideways Out", new Vector2(545, -1065));

            AddElement("Info4", _ =>
            {
                const string text = "Sideways morphing is based on the joint rotating away from the calibrated horizontal angle (plus offset).";
                var uiDynamic = CreateTextField(text, new Vector2(10, -1065), new Vector2(-555, 118));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });
        }

        void BuildTittyMagicUI()
        {
            AddSlider(_forceMorphHandler.baseJsf, _ => CreateSlider("Base Multiplier", "F2", new Vector2(10, -280)));

            AddMultiplierSlider(_forceMorphHandler.upJsf, "Up", new Vector2(545, -280));

            AddElement("Info1", _ =>
            {
                string text = new StringBuilder()
                    .AppendLine("Control how much breast shape is adjusted dynamically with morphs.")
                    .AppendLine()
                    .Append("The amount of morphing is based on the breast's movement away from its neutral position relative to the chest.")
                    .AppendLine("The neutral position is estimated as the position when the person is in an upright pose and only the force of gravity is applied.")
                    .AppendLine()
                    .Append("Anything that causes the breast to move will cause morphing: collision, gravity or any kind of animation.")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -920), new Vector2(-555, 620));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddMultiplierSlider(_forceMorphHandler.depthOutJsf, "Depth Out", new Vector2(545, -410));
            AddMultiplierSlider(_forceMorphHandler.depthInJsf, "Depth In", new Vector2(545, -540));
            AddMultiplierSlider(_forceMorphHandler.sidewaysInJsf, "Sideways In", new Vector2(545, -670));
            AddMultiplierSlider(_forceMorphHandler.sidewaysOutJsf, "Sideways Out", new Vector2(545, -800));

            AddElement("Other Settings", name =>
            {
                var uiDynamic = CreateTextField(name.Bold(), new Vector2(10, -1105), new Vector2(-555, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            AddElement("Info2", _ =>
            {
                const string text = "Expand nipple morphs and harden nipple physics.";
                var uiDynamic = CreateTextField(text, new Vector2(10, -1275), new Vector2(-555, 180));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddSlider(
                script.nippleErectionHandler.NippleErectionJsf,
                name => CreateSlider(name, "F2", new Vector2(545, -1215))
            );
        }

        void AddMultiplierSlider(JSONStorableFloat multiplierJsf, string label, Vector2 position) =>
            AddSlider(multiplierJsf, _ => CreateSlider(label, "F2", position));

        internal override void UpdateDiagnostics(bool atInterval)
        {
            if(!isBuilt || !atInterval)
            {
                return;
            }

            UpdateSliderColor(_forceMorphHandler.upJsf);
            if(script.IsBootyMagic)
            {
                UpdateSliderColor(_forceMorphHandler.downJsf);
            }

            UpdateSliderColor(_forceMorphHandler.depthOutJsf);
            UpdateSliderColor(_forceMorphHandler.depthInJsf);
            UpdateSliderColor(_forceMorphHandler.sidewaysInJsf);
            UpdateSliderColor(_forceMorphHandler.sidewaysOutJsf);
        }

        void UpdateSliderColor(JSONStorableFloat storable)
        {
            var slider = GetElement(storable.name)?.GetUIDynamicAs<UIDynamicSlider>();
            if(slider)
            {
                var images = slider.slider.gameObject.transform.GetComponentsInChildren<Image>();
                var fillImage = images.First(image => image.name == "Fill");
                var handleImage = images.First(image => image.name == "Handle");
                var color = MultiplierSliderColor(_forceMorphHandler.baseJsf.val * storable.val);
                fillImage.color = color;
                handleImage.color = color;
            }
        }

        static Color MultiplierSliderColor(float value) => value <= 1
            ? Color.Lerp(new Color(1, 1, 1, 0.25f), Color.white, value)
            : Color.Lerp(Color.white, new Color(1.0f, 0.2f, 0.2f), (value - 1) / 3);
    }
}
