using NAT.Common.Handlers;
using NAT.Common.Models;
using UnityEngine;

namespace NAT.Common.UI.Windows
{
    sealed class ParameterWindow : Window
    {
        readonly JointPhysicsHandler _jointPhysicsHandler;
        readonly JointPhysicsParamPair _paramPair;
        readonly JointPhysicsParam _param;
        readonly ParamValueInfo _paramValueInfo;

        internal ParameterWindow(ScriptBase script, JointPhysicsParamPair paramPair) : base(script, paramPair.name)
        {
            _jointPhysicsHandler = script.jointPhysicsHandler;
            _paramPair = paramPair;
            _param = _paramPair.Left;
            _paramValueInfo = new ParamValueInfo(_paramPair);
        }

        protected override void Build()
        {
            AddElement("Parameter Info", _ =>
            {
                var uiDynamic = CreateTextField(_paramPair.infoText, new Vector2(10, -435), new Vector2(-555, 280));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            var offsetStorable = _paramPair == _jointPhysicsHandler.twistAngleTarget
                ? script.gravityPhysicsHandler.twistAngleTargetJsf
                : _param.offsetJsf;
            AddSlider(offsetStorable, _ => CreateSlider("Offset", _paramPair.ValueFormat, new Vector2(545, -265)));

            AddElement("Value", _ =>
            {
                var uiDynamic = CreateTextField("", new Vector2(545, -330), new Vector2(-555, 45));
                uiDynamic.UItext.fontSize = 28;
                _paramValueInfo.textField = uiDynamic;
                return uiDynamic;
            });

            if(_param.HasAdjustableGravityPhysics())
            {
                AddElement("Gravity Physics", name =>
                {
                    var uiDynamic = CreateTextField(name.Bold(), new Vector2(10, -470), new Vector2(-555, 60));
                    uiDynamic.UItext.fontSize = 30;
                    uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                    return uiDynamic;
                });

                AddElement("Gravity Targets Info", _ =>
                {
                    var uiDynamic = CreateTextField("", new Vector2(10, -520), new Vector2(-555, 50));
                    uiDynamic.UItext.fontSize = 28;
                    if(script.IsBootyMagic)
                    {
                        uiDynamic.text = "Left glute gravity target values:";
                    }
                    else if(script.IsTittyMagic)
                    {
                        uiDynamic.text = "Left breast gravity target values:";
                    }

                    return uiDynamic;
                });

                AddElement("Gravity Targets Values", _ =>
                {
                    var uiDynamic = CreateTextField("", new Vector2(10, -690), new Vector2(-555, 200));
                    uiDynamic.UItext.fontSize = 28;
                    _paramValueInfo.gravityTargetsField = uiDynamic;
                    return uiDynamic;
                });

                const float sliderSpacing = 130;
                var keys = _param.gravityCalculators.Keys.ToList();

                for(int i = 0; i < keys.Count; i++)
                {
                    string key = keys[i];
                    var config = _param.gravityCalculators[key];
                    float posY = -590 - i * sliderSpacing;
                    AddSlider(config.multiplierJsf, _ => CreateSlider($"{key} Gravity", "F2", new Vector2(545, posY)));
                }
            }
        }

        internal override void UpdateDiagnostics(bool atInterval)
        {
            if(!isBuilt || !atInterval)
            {
                return;
            }

            // TODO separate value indicators for different sources e.g. clothing
            _paramValueInfo.Update(infoSb);
            _paramValueInfo.UpdateGravityTargets(infoSb, script.jointPhysicsHandler.normalizedMass, script.softness);
        }
    }
}
