using System.Text;
using UnityEngine;

namespace NAT.Common.UI.Windows
{
    sealed class SettingsWindow : Window
    {
        readonly Bootstrap _bootstrap;

        internal SettingsWindow(ScriptBase script) : base(script, nameof(SettingsWindow))
        {
            _bootstrap = script.bootstrap;
        }

        protected override void Build()
        {
            BuildUserPreferencesAreaLeft();
            BuildUserPreferencesAreaRight();
            BuildPluginSettingsArea();
        }

        void BuildUserPreferencesAreaLeft()
        {
            AddElement("User Preferences", name =>
            {
                var uiDynamic = CreateTextField(name.Bold(), new Vector2(10, -225), new Vector2(-555, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            AddElement("userPreferencesInfo", _ =>
            {
                string text = new StringBuilder()
                    .AppendFormat("These settings are saved to file and apply to all {0} instances.", Constant.NATURALIS)
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -260), new Vector2(-20, 40));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddToggle(_bootstrap.CalibrateOnUIClosedJsb, name => CreateToggle(name, new Vector2(10, -320), -555));
            AddElement("calibrateOnUIClosedInfo", _ =>
            {
                string text = new StringBuilder()
                    .Append("If a calibration is pending, it will trigger when you navigate away from the")
                    .AppendFormat(" {0} or {1} plugin UI", Constant.TITTY_MAGIC, Constant.BOOTY_MAGIC)
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -430), new Vector2(-555, 100));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddToggle(_bootstrap.CalibrateOnPresetLoadedJsb, name => CreateToggle(name, new Vector2(10, -490), -555));
            AddElement("calibrateOnPresetLoadedInfo", _ =>
            {
                const string text = "Calibrate immediately after loading a preset file. Puts to pending instead if disabled.";
                var uiDynamic = CreateTextField(text, new Vector2(10, -570), new Vector2(-555, 70));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddToggle(_bootstrap.CheckForSizeChangesJsb, name => CreateToggle(name, new Vector2(10, -630), -555));
            AddElement("checkForSizeChangesInfo", _ =>
            {
                const string text = "Monitor for size changes to breasts/glutes caused by non-pose morphs and atom scale. A change puts calibration to pending.";
                var uiDynamic = CreateTextField(text, new Vector2(10, -740), new Vector2(-555, 100));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddToggle(_bootstrap.ForceEnableAtomSoftPhysicsJsb, name => CreateToggle(name, new Vector2(10, -800), -555));
            AddElement("forceEnableAtomSoftPhysicsInfo", _ =>
            {
                const string text = "Automatically enable soft physics in Control & Physics 1 and prevent it from being toggled off.";
                var uiDynamic = CreateTextField(text, new Vector2(10, -910), new Vector2(-555, 100));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });
        }

        void BuildUserPreferencesAreaRight()
        {
            AddElement("openDataDir", _ =>
            {
                const string label = "Open plugin data directory\nin Windows Explorer";
                var uiDynamic = CreateButton(label, new Vector2(545, -175), new Vector2(-555, 90f));
                uiDynamic.AddListener(FileUtils.OpenDataDirInExplorer);
                return uiDynamic;
            });

            AddTextField(_bootstrap.SavedPrefsInfoJss, _ =>
            {
                var uiDynamic = CreateTextField("", new Vector2(545, -230), new Vector2(-555, 50));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddToggle(_bootstrap.GlutesLdpDefaultEnabledJsb, name => CreateToggle(name, new Vector2(545, -320), -555));
            AddToggle(_bootstrap.BreastsLdpDefaultEnabledJsb, name => CreateToggle(name, new Vector2(545, -380), -555));

            AddElement("ldpDefaultEnabledInfo", _ =>
            {
                var sb = new StringBuilder()
                    .Append("Enable ")
                    .AppendBold("Local Distortion Physics")
                    .Append(" by default when plugin is added and when soft physics is enabled. LDP can cause a noticeable framerate drop.");
                var uiDynamic = CreateTextField(sb.ToString(), new Vector2(545, -510), new Vector2(-555, 120));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddPopup(_bootstrap.LdpUpdateRateJssc, _ => CreateScrollablePopup(_bootstrap.LdpUpdateRateJssc.label, new Vector2(545, -625), new Vector2(-555, 100)));
            AddElement("ldpUpdateRateInfo", _ =>
            {
                const string text = "A higher Local Distortion Physics update rate is more responsive, but can reduce frame rate.";
                var uiDynamic = CreateTextField(text, new Vector2(545, -740), new Vector2(-555, 100));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddPopup(_bootstrap.PhysicsQualityJssc,
                _ => CreateScrollablePopup(_bootstrap.PhysicsQualityJssc.label, new Vector2(545, -830), new Vector2(-555, 100)));
            AddElement("physicsQualityInfo", _ =>
            {
                const string text =
                    "Physics quality of breast/glute joints and hard colliders. 5 is equivalent to High Quality Physics. Warning: High performance cost!";
                var uiDynamic = CreateTextField(text, new Vector2(545, -940), new Vector2(-555, 100));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });
        }

        void BuildPluginSettingsArea()
        {
            AddElement("Plugin Settings", name =>
            {
                var uiDynamic = CreateTextField(name.Bold(), new Vector2(10, -1000), new Vector2(-555, 60));
                uiDynamic.UItext.fontSize = 30;
                uiDynamic.UItext.alignment = TextAnchor.LowerCenter;
                return uiDynamic;
            });

            AddElement("pluginSettingsInfo", _ =>
            {
                string text = new StringBuilder()
                    .AppendFormat("These settings save with the current {0} instance.", Constant.NATURALIS)
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -1030), new Vector2(-20, 40));
                uiDynamic.UItext.fontSize = 28;
                return uiDynamic;
            });

            AddToggle(_bootstrap.FreezeMotionSoundJsb, name => CreateToggle(name, new Vector2(10, -1090), -555));
            AddElement("Freeze Motion Sound Info", _ =>
            {
                string text = new StringBuilder()
                    .Append("Enable the Freeze/Motion Sound toggle (in the below toolbar) during calibration.")
                    .Append(" This prevents all animation from interfering with the calibration, but also disables audio.")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(10, -1230), new Vector2(-555, 130));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });

            AddToggle(_bootstrap.PauseSceneAnimationJsb, name => CreateToggle(name, new Vector2(545, -1090), -555));
            AddElement("Pause Scene Animation Info", _ =>
            {
                string text = new StringBuilder()
                    .Append("Pause scene animation during calibration. Warning: this has no effect on other animation sources")
                    .Append(" (Animation Pattern, Timeline or other plugin, Cycle Forces, etc.")
                    .ToString();
                var uiDynamic = CreateTextField(text, new Vector2(545, -1230), new Vector2(-555, 130));
                uiDynamic.UItext.fontSize = 26;
                return uiDynamic;
            });
        }

        protected override void PostShow()
        {
            GetElement(_bootstrap.PauseSceneAnimationJsb.name)
                .SetActiveStyle(!_bootstrap.FreezeMotionSoundJsb.val, true);
        }
    }
}
