using SimpleJSON;
using System.Collections.Generic;
using UnityEngine;

namespace NAT.Common
{
    static class JSONUtils
    {
        internal static void RenameKey(JSONClass jc, string from, string to)
        {
            if(jc.HasKey(from))
            {
                jc[to] = jc[from];
                jc.Remove(from);
            }
        }

        internal static JSONClass SerializeQuaternion(Quaternion quaternion)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = quaternion.x },
                ["y"] = { AsFloat = quaternion.y },
                ["z"] = { AsFloat = quaternion.z },
                ["w"] = { AsFloat = quaternion.w },
            };
            return jc;
        }

        internal static Quaternion DeserializeQuaternion(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Quaternion(
                jc["x"].AsFloat,
                jc["y"].AsFloat,
                jc["z"].AsFloat,
                jc["w"].AsFloat
            );
        }

        internal static JSONClass SerializeVector3(Vector3 vector)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = vector.x },
                ["y"] = { AsFloat = vector.y },
                ["z"] = { AsFloat = vector.z },
            };
            return jc;
        }

        internal static Vector3 DeserializeVector3(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Vector3(
                jc["x"].AsFloat,
                jc["y"].AsFloat,
                jc["z"].AsFloat
            );
        }

        internal static JSONClass SerializeVector2(Vector2 vector)
        {
            var jc = new JSONClass
            {
                ["x"] = { AsFloat = vector.x },
                ["y"] = { AsFloat = vector.y },
            };
            return jc;
        }

        internal static Vector2 DeserializeVector2(JSONNode jsonNode)
        {
            var jc = jsonNode.AsObject;
            return new Vector3(
                jc["x"].AsFloat,
                jc["y"].AsFloat
            );
        }

        internal static IEnumerable<JSONNode> GetJSONArrayEnumerable(JSONClass jc, string arrayKey)
        {
            if(jc.HasKey(arrayKey))
            {
                foreach(JSONNode node in jc[arrayKey].AsArray)
                {
                    yield return node;
                }
            }
        }
    }
}
