using NAT.Common;
using NAT.Common.Handlers;
using NAT.Common.Handlers.Configs;
using NAT.Common.Models;
using NAT.Common.Storables;
using UnityEngine;

namespace NAT.TM.Handlers
{
    sealed class ChestClothingHandler : ClothingHandler
    {
        internal ChestClothingHandler(ScriptBase script) : base(script)
        {
            ClothingProfilesDir = "ChestClothingProfiles";
            validRegionTags = new[] { "full body", "torso" };
            invalidRegionTags = new[] { "hip", "arms", "feet", "hands", "head", "legs", "neck" };
            presetHandler = new ClothingPresetHandler(Script, this, ClothingProfilesDir);
            CreateMorphConfigs();
            LoadSettings();
            initialized = true;
        }

        void CreateMorphConfigs()
        {
            const string id = "Reduce Size";
            var config = NewMorphConfig(id);
            config.offsetsCompensation = new Vector3(0f, 0f, -0.0030f);
            morphConfigs[id] = config;
            AddMorphConfigCallbacks();
        }

        ClothingMorphConfig NewMorphConfig(string id) => new ClothingMorphConfig(
            Person.GetBreastMorph($"Clothing/{id} L"),
            Person.GetBreastMorph($"Clothing/{id} R"),
            new StorableFloat(id, 0.000f, 0.000f, 2.000f)
        );

        internal override void SetMultipliers()
        {
            float massCurve = Calc.InverseSmoothStep(1 - jointPhysicsHandler.normalizedRealMass, 0.60f, 0.95f);
            upDownMassMultiplier = Mathf.Lerp(0.5f, 1.0f, massCurve);
            sidewaysMassMultiplier = Mathf.Lerp(0.17f, 1.0f, massCurve);
            inOutMassMultiplier = Mathf.Lerp(0.17f, 1.0f, massCurve);
        }

        protected override void SyncJointPhysicsAdjustments()
        {
            base.SyncJointPhysicsAdjustments();
            jointPhysicsHandler.rightLeftSpring.Left.SetClothingAdjustment(CalculateHoldAmount(LeftYXHoldPct));
            jointPhysicsHandler.rightLeftSpring.Right.SetClothingAdjustment(CalculateHoldAmount(RightYXHoldPct));
            jointPhysicsHandler.rightLeftDamper.Left.SetClothingAdjustment(0.01f * LeftYXDampeningPct);
            jointPhysicsHandler.rightLeftDamper.Right.SetClothingAdjustment(0.01f * RightYXDampeningPct);
        }

        protected override ClothingAdjustmentProfile GetProfile(ClothingItem item) => item.chestProfile;
        protected override void SetProfile(ClothingItem item, ClothingAdjustmentProfile profile) => item.chestProfile = profile;
    }
}
