﻿using NAT.Common;
using NAT.Common.Contexts;
using NAT.Common.Handlers;
using NAT.Common.Handlers.Configs;
using NAT.Common.Models;
using NAT.Common.Storables;

namespace NAT.TM.Handlers
{
    sealed class NippleErectionHandler : Handler
    {
        readonly SoftPhysicsContext _leftContext;
        readonly SoftPhysicsContext _rightContext;

        MorphWrapper _morph;
        internal StorableFloat NippleErectionJsf { get; }
        PhysicsCalculator _springCalculator;
        PhysicsCalculator _damperCalculator;

        internal NippleErectionHandler(ScriptBase script) : base(script)
        {
            _leftContext = Script.softPhysicsHandler.leftContext;
            _rightContext = Script.softPhysicsHandler.rightContext;
            NippleErectionJsf = new StorableFloat("Nipple Erection", 0.00f, 0.00f, 1.00f)
            {
                setCallbackFunction = _ => UpdateNipple(),
            };
            LoadSettings();
        }

        protected override void LoadSettings()
        {
            _morph = new MorphWrapper(Person.GetBreastMorph("TM_NippleErection"));
            _springCalculator = new PhysicsCalculator
            {
                primaryConfig = new MultiplierConfig
                {
                    baseValue = 0.50f,
                    multiplierA = 0.75f,
                    multiplierB = 0,
                    curveA = x => 1 - x,
                },
            };
            _damperCalculator = new PhysicsCalculator
            {
                primaryConfig = new MultiplierConfig
                {
                    baseValue = 0.50f,
                    multiplierA = 0.75f,
                    multiplierB = 0,
                    curveA = x => 1 - x,
                },
            };
        }

        internal override void PreRegisterParams()
        {
            Script.PreRegisterMorphsParam(NippleErectionJsf);
        }

        internal void UpdateNipple()
        {
            if(!Script.IsEnabledAndActive())
            {
                return;
            }

            float mass = Script.jointPhysicsHandler.normalizedMass;
            float softness = Script.softness;
            _morph.SetValue(NippleErectionJsf.val);
            if(Script.softPhysicsHandler.softPhysicsOn)
            {
                float springMultiplier = 1 + _springCalculator.CalculateDynamic(NippleErectionJsf.val, mass, softness, true);
                float damperMultiplier = 1 + _damperCalculator.CalculateDynamic(NippleErectionJsf.val, mass, softness, true);
                _leftContext.nippleSoftJoint.SetAdditionalBaseMultiplier(FAT_SPRING.NAME, springMultiplier);
                _leftContext.nippleSoftJoint.SetAdditionalBaseMultiplier(FAT_DAMPER.NAME, damperMultiplier);
                _rightContext.nippleSoftJoint.SetAdditionalBaseMultiplier(FAT_SPRING.NAME, springMultiplier);
                _rightContext.nippleSoftJoint.SetAdditionalBaseMultiplier(FAT_DAMPER.NAME, damperMultiplier);
            }
        }

        internal void Reset()
        {
            _morph.Reset();
            // physics values are not reset here but by SoftPhysicsHandler
        }

        internal override void Disable()
        {
            Reset();
        }
    }
}
