using System;
using System.Collections.Generic;

namespace NAT.TM
{
    static class VertexIndices
    {
        // areola vert, 2 main verts, 1-3 outer verts
        internal static IEnumerable<int[]> CenterToOuterVerts(string side)
        {
            switch(side)
            {
                case Side.LEFT:
                    return new[]
                    {
                        new[] { 7944, 8935, 8852, 8854 },
                        new[] { 7946, 8936, 8849, 8837 },
                        new[] { 7965, 8931, 7248, 8 },
                        new[] { 7950, 8928, 7245, 2403, 2871 },
                        new[] { 7952, 8929, 7243, 2402, 2406, 7224 },
                        new[] { 7966, 8925, 7230, 2409, 2408, 7223 },
                        new[] { 7964, 8947, 7232, 2411, 2410, 7221 },
                        new[] { 7958, 8945, 7234, 2416, 2415, 7219 },
                        new[] { 7954, 8941, 7236, 138, 135, 7216 },
                        new[] { 7947, 8942, 7238, 2591, 2592 },
                        new[] { 7959, 8938, 7240, 2597, 2598 },
                        new[] { 7945, 8933, 7241, 16 },
                    };
                case Side.RIGHT:
                    return new[]
                    {
                        new[] { 18622, 19601, 19523, 19525 },
                        new[] { 18624, 19602, 19520, 19508 },
                        new[] { 18643, 19597, 17954, 10936 },
                        new[] { 18628, 19594, 17951, 13233, 13672 },
                        new[] { 18630, 19595, 17949, 13232, 13236, 17930 },
                        new[] { 18644, 19591, 17936, 13239, 13238, 17929 },
                        new[] { 18642, 19613, 17938, 13241, 13240, 17927 },
                        new[] { 18636, 19611, 17940, 13246, 13245, 17925 },
                        new[] { 18632, 19607, 17942, 11066, 11063, 17923 },
                        new[] { 18625, 19608, 17944, 13409, 13410 },
                        new[] { 18637, 19604, 17946, 13413, 13414 },
                        new[] { 18623, 19599, 17947, 10944 },
                    };
                default: throw new ArgumentException($"Invalid side {side}");
            }
        }

        // connected verts on the same "ring", for calculating averages from
        internal static Dictionary<int, int[]> AdjacentVerts(string side)
        {
            switch(side)
            {
                case Side.LEFT:
                    return new Dictionary<int, int[]>
                    {
                        // main ring 1
                        { 8935, new[] { 8934, 8937 } },
                        { 8937, new[] { 8936, 8935 } },
                        { 8936, new[] { 8937, 8940 } },
                        { 8940, new[] { 8936, 8931 } },
                        { 8931, new[] { 8940, 8926 } },
                        { 8926, new[] { 8928, 8931 } },
                        { 8928, new[] { 8930, 8926 } },
                        { 8930, new[] { 8928, 8929 } },
                        { 8929, new[] { 8930, 8927 } },
                        { 8927, new[] { 8925, 8929 } },
                        { 8925, new[] { 8927, 8948 } },
                        { 8948, new[] { 8947, 8925 } },
                        { 8947, new[] { 8946, 8948 } },
                        { 8946, new[] { 8945, 8947 } },
                        { 8945, new[] { 8944, 8946 } },
                        { 8944, new[] { 8945, 8941 } },
                        { 8941, new[] { 8943, 8944 } },
                        { 8943, new[] { 8941, 8942 } },
                        { 8942, new[] { 8939, 8943 } },
                        { 8939, new[] { 8938, 8942 } },
                        { 8938, new[] { 8939, 8932 } },
                        { 8932, new[] { 8933, 8938 } },
                        { 8933, new[] { 8932, 8934 } },
                        { 8934, new[] { 8933, 8935 } },
                        // main ring 2
                        { 8852, new[] { 8848, 8851 } },
                        { 8851, new[] { 8849, 8852 } },
                        { 8849, new[] { 8850, 8851 } },
                        { 8850, new[] { 7248, 8849 } },
                        { 7248, new[] { 8850, 7246 } },
                        { 7246, new[] { 7245, 7248 } },
                        { 7245, new[] { 7244, 7246 } },
                        { 7244, new[] { 7243, 7245 } },
                        { 7243, new[] { 7244, 7247 } },
                        { 7247, new[] { 7243, 7230 } },
                        { 7230, new[] { 7231, 7247 } },
                        { 7231, new[] { 7230, 7232 } },
                        { 7232, new[] { 7233, 7231 } },
                        { 7233, new[] { 7232, 7234 } },
                        { 7234, new[] { 7233, 7235 } },
                        { 7235, new[] { 7234, 7236 } },
                        { 7236, new[] { 7235, 7237 } },
                        { 7237, new[] { 7236, 7238 } },
                        { 7238, new[] { 7237, 7239 } },
                        { 7239, new[] { 7238, 7240 } },
                        { 7240, new[] { 7242, 7239 } },
                        { 7242, new[] { 7240, 7241 } },
                        { 7241, new[] { 7242, 8848 } },
                        { 8848, new[] { 8852, 7241 } },
                        // outer ring 1
                        { 8, new[] { 2404 } },
                        { 16, new[] { 17, 8839 } },
                        { 17, new[] { 16, 2597 } },
                        { 137, new[] { 2591, 138 } },
                        { 138, new[] { 137, 2419 } },
                        { 2401, new[] { 2403, 2402 } },
                        { 2402, new[] { 2401, 2405 } },
                        { 2403, new[] { 2401, 2404 } },
                        { 2404, new[] { 8, 2403 } },
                        { 2405, new[] { 2402, 2409 } },
                        { 2409, new[] { 2413, 2405 } },
                        { 2411, new[] { 2413, 2414 } },
                        { 2413, new[] { 2409, 2411 } },
                        { 2414, new[] { 2411, 2416 } },
                        { 2416, new[] { 2419, 2414 } },
                        { 2419, new[] { 2416, 138 } },
                        { 2591, new[] { 137, 2596 } },
                        { 2596, new[] { 2597, 2591 } },
                        { 2597, new[] { 17, 2596 } },
                        { 8837, new[] { 8838, 8853 } },
                        { 8838, new[] { 8837 } },
                        { 8839, new[] { 16, 8854 } },
                        { 8853, new[] { 8837, 8854 } },
                        { 8854, new[] { 8853, 8839 } },
                        // outer ring 2
                        { 93, new[] { 2406, 2871 } },
                        { 135, new[] { 136, 2418 } },
                        { 136, new[] { 2592, 135 } },
                        { 2406, new[] { 93, 2407 } },
                        { 2407, new[] { 2406, 2408 } },
                        { 2408, new[] { 2412, 2407 } },
                        { 2410, new[] { 2412, 2417 } },
                        { 2412, new[] { 2408, 2410 } },
                        { 2415, new[] { 2417, 2418 } },
                        { 2417, new[] { 2410, 2415 } },
                        { 2418, new[] { 135, 2415 } },
                        { 2590, new[] { 2598 } },
                        { 2592, new[] { 136, 2594 } },
                        { 2594, new[] { 2592, 2598 } },
                        { 2598, new[] { 2594, 2590 } },
                        { 2871, new[] { 93 } },
                        // outer ring 3
                        { 7216, new[] { 7218 } },
                        { 7218, new[] { 7216, 7219 } },
                        { 7219, new[] { 7218, 7220 } },
                        { 7220, new[] { 7219, 7221 } },
                        { 7221, new[] { 7220, 7222 } },
                        { 7222, new[] { 7221, 7223 } },
                        { 7223, new[] { 7222, 7226 } },
                        { 7224, new[] { 7225, 7226 } },
                        { 7225, new[] { 7224 } },
                        { 7226, new[] { 7224, 7223 } },
                    };
                case Side.RIGHT:
                    return new Dictionary<int, int[]>
                    {
                        // main ring 1
                        { 19601, new[] { 19600, 19603 } },
                        { 19603, new[] { 19602, 19601 } },
                        { 19602, new[] { 19603, 19606 } },
                        { 19606, new[] { 19602, 19597 } },
                        { 19597, new[] { 19606, 19592 } },
                        { 19592, new[] { 19594, 19597 } },
                        { 19594, new[] { 19596, 19592 } },
                        { 19596, new[] { 19594, 19595 } },
                        { 19595, new[] { 19596, 19593 } },
                        { 19593, new[] { 19591, 19595 } },
                        { 19591, new[] { 19593, 19614 } },
                        { 19614, new[] { 19613, 19591 } },
                        { 19613, new[] { 19612, 19614 } },
                        { 19612, new[] { 19611, 19613 } },
                        { 19611, new[] { 19610, 19612 } },
                        { 19610, new[] { 19611, 19607 } },
                        { 19607, new[] { 19609, 19610 } },
                        { 19609, new[] { 19607, 19608 } },
                        { 19608, new[] { 19605, 19609 } },
                        { 19605, new[] { 19604, 19608 } },
                        { 19604, new[] { 19605, 19598 } },
                        { 19598, new[] { 19599, 19604 } },
                        { 19599, new[] { 19598, 19600 } },
                        { 19600, new[] { 19599, 19601 } },
                        // main ring 2
                        { 19523, new[] { 19519, 19522 } },
                        { 19522, new[] { 19520, 19523 } },
                        { 19520, new[] { 19521, 19522 } },
                        { 19521, new[] { 17954, 19520 } },
                        { 17954, new[] { 19521, 17952 } },
                        { 17952, new[] { 17951, 17954 } },
                        { 17951, new[] { 17950, 17952 } },
                        { 17950, new[] { 17949, 17951 } },
                        { 17949, new[] { 17950, 17953 } },
                        { 17953, new[] { 17949, 17936 } },
                        { 17936, new[] { 17937, 17953 } },
                        { 17937, new[] { 17936, 17938 } },
                        { 17938, new[] { 17939, 17937 } },
                        { 17939, new[] { 17938, 17940 } },
                        { 17940, new[] { 17939, 17941 } },
                        { 17941, new[] { 17940, 17942 } },
                        { 17942, new[] { 17941, 17943 } },
                        { 17943, new[] { 17942, 17944 } },
                        { 17944, new[] { 17943, 17945 } },
                        { 17945, new[] { 17944, 17946 } },
                        { 17946, new[] { 17948, 17945 } },
                        { 17948, new[] { 17946, 17947 } },
                        { 17947, new[] { 17948, 19519 } },
                        { 19519, new[] { 19523, 17947 } },
                        // outer ring 1
                        { 10936, new[] { 13234 } },
                        { 10944, new[] { 10945, 19510 } },
                        { 10945, new[] { 10944, 13413 } },
                        { 11065, new[] { 13409, 11066 } },
                        { 11066, new[] { 11065, 13249 } },
                        { 13231, new[] { 13233, 13232 } },
                        { 13232, new[] { 13231, 13235 } },
                        { 13233, new[] { 13231, 13234 } },
                        { 13234, new[] { 10936, 13233 } },
                        { 13235, new[] { 13232, 13239 } },
                        { 13239, new[] { 13243, 13235 } },
                        { 13241, new[] { 13243, 13244 } },
                        { 13243, new[] { 13239, 13241 } },
                        { 13244, new[] { 13241, 13246 } },
                        { 13246, new[] { 13249, 13244 } },
                        { 13249, new[] { 13246, 11066 } },
                        { 13409, new[] { 11065, 13412 } },
                        { 13412, new[] { 13413, 13409 } },
                        { 13413, new[] { 10945, 13412 } },
                        { 19508, new[] { 19509, 19524 } },
                        { 19509, new[] { 19508 } },
                        { 19510, new[] { 10944, 19525 } },
                        { 19524, new[] { 19508, 19525 } },
                        { 19525, new[] { 19524, 19510 } },
                        // outer ring 2
                        { 11021, new[] { 13236, 13672 } },
                        { 11063, new[] { 11064, 13248 } },
                        { 11064, new[] { 13410, 11063 } },
                        { 13236, new[] { 11021, 13237 } },
                        { 13237, new[] { 13236, 13238 } },
                        { 13238, new[] { 13242, 13237 } },
                        { 13240, new[] { 13242, 13247 } },
                        { 13242, new[] { 13238, 13240 } },
                        { 13245, new[] { 13247, 13248 } },
                        { 13247, new[] { 13240, 13245 } },
                        { 13248, new[] { 11063, 13245 } },
                        { 13408, new[] { 13414 } },
                        { 13410, new[] { 11064, 13411 } },
                        { 13411, new[] { 13410, 13414 } },
                        { 13414, new[] { 13411, 13408 } },
                        { 13672, new[] { 11021 } },
                        // outer ring 3
                        { 17923, new[] { 17924 } },
                        { 17924, new[] { 17923, 17925 } },
                        { 17925, new[] { 17924, 17926 } },
                        { 17926, new[] { 17925, 17927 } },
                        { 17927, new[] { 17926, 17928 } },
                        { 17928, new[] { 17927, 17929 } },
                        { 17929, new[] { 17928, 17932 } },
                        { 17930, new[] { 17931, 17932 } },
                        { 17931, new[] { 17930 } },
                        { 17932, new[] { 17930, 17929 } },
                    };
                default: throw new ArgumentException($"Invalid side {side}");
            }
        }

        internal static Dictionary<int, int[]> NearbyVerts(string side)
        {
            switch(side)
            {
                case Side.LEFT:
                    return new Dictionary<int, int[]>
                    {
                        // nipple
                        { 0, new[] { 7944, 7965, 7952, 7964, 7954, 7959 } },
                        // areola
                        { 7944, new[] { 7945, 7946, 8935 } },
                        { 7946, new[] { 7944, 7965, 8936 } },
                        { 7965, new[] { 7946, 7950, 8931 } },
                        { 7950, new[] { 7952, 7965, 8928 } },
                        { 7952, new[] { 7950, 7966, 8929 } },
                        { 7966, new[] { 7952, 7964, 8925 } },
                        { 7964, new[] { 7958, 7966, 8947 } },
                        { 7958, new[] { 7954, 7964, 8945 } },
                        { 7954, new[] { 7947, 7958, 8941 } },
                        { 7947, new[] { 7954, 7959, 8942 } },
                        { 7959, new[] { 7945, 7947, 8938 } },
                        { 7945, new[] { 7944, 7959, 8933 } },
                        // main ring 1
                        { 8935, new[] { 7944, 8934, 8937, 8852 } },
                        { 8937, new[] { 7944, 7946, 8936, 8935, 8851 } },
                        { 8936, new[] { 7946, 8937, 8940, 8849 } },
                        { 8940, new[] { 7946, 7965, 8936, 8931, 8850 } },
                        { 8931, new[] { 7965, 8940, 8926, 7248 } },
                        { 8926, new[] { 7965, 7950, 8928, 8931, 7246 } },
                        { 8928, new[] { 7950, 8930, 8926, 7245 } },
                        { 8930, new[] { 7950, 7952, 8928, 8929, 7244 } },
                        { 8929, new[] { 7952, 8930, 8927, 7243 } },
                        { 8927, new[] { 7952, 7966, 8925, 8929, 7247 } },
                        { 8925, new[] { 7966, 8927, 8948, 7230 } },
                        { 8948, new[] { 7966, 7964, 8947, 8925, 7231 } },
                        { 8947, new[] { 7964, 8946, 8948, 7232 } },
                        { 8946, new[] { 7964, 7958, 8945, 8947, 7233 } },
                        { 8945, new[] { 7958, 8944, 8946, 7234 } },
                        { 8944, new[] { 7958, 7954, 8945, 8941, 7235 } },
                        { 8941, new[] { 7954, 8943, 8944, 7236 } },
                        { 8943, new[] { 7954, 7947, 8941, 8942, 7237 } },
                        { 8942, new[] { 7947, 8939, 8943, 7238 } },
                        { 8939, new[] { 7947, 7959, 8938, 8942, 7239 } },
                        { 8938, new[] { 7959, 8939, 8932, 7240 } },
                        { 8932, new[] { 7959, 7945, 8933, 8938, 7242 } },
                        { 8933, new[] { 7945, 8932, 8934, 7241 } },
                        { 8934, new[] { 7945, 7944, 8933, 8935, 8848 } },
                        // main ring 2
                        { 8852, new[] { 8839, 8848, 8851, 8853, 8854, 8935 } },
                        { 8851, new[] { 8937, 8849, 8852, 8853, 8854, 8837 } },
                        { 8849, new[] { 8837, 8838, 8850, 8851, 8853, 8936 } },
                        { 8850, new[] { 7248, 8837, 8838, 8849, 8936, 8940 } },
                        { 7248, new[] { 8, 2404, 7246, 8850, 8931, 8940 } },
                        { 7246, new[] { 8926, 8, 2403, 2404, 7245, 7248 } },
                        { 7245, new[] { 2401, 2403, 2404, 7244, 7246, 8928 } },
                        { 7244, new[] { 2401, 2402, 2403, 7243, 7245, 8930 } },
                        { 7243, new[] { 8929, 2401, 2402, 7244, 7247, 2405 } },
                        { 7247, new[] { 2402, 2405, 2409, 7230, 7243, 8927 } },
                        { 7230, new[] { 2405, 2409, 2413, 7231, 7247, 8925 } },
                        { 7231, new[] { 8948, 2409, 2411, 2413, 7230, 7232 } },
                        { 7232, new[] { 2411, 2413, 2414, 7231, 7233, 8947 } },
                        { 7233, new[] { 8946, 7232, 2416, 7234, 2411, 2414 } },
                        { 7234, new[] { 2414, 2416, 2419, 7233, 7235, 8945 } },
                        { 7235, new[] { 138, 2416, 2419, 7234, 7236, 8944 } },
                        { 7236, new[] { 8941, 137, 138, 7235, 2419, 7237 } },
                        { 7237, new[] { 137, 138, 2591, 7236, 7238, 8943 } },
                        { 7238, new[] { 8942, 137, 2596, 7237, 7239, 2591 } },
                        { 7239, new[] { 2591, 2596, 2597, 7238, 7240, 8939 } },
                        { 7240, new[] { 17, 2596, 2597, 7239, 7242, 8938 } },
                        { 7242, new[] { 16, 17, 2597, 7240, 7241, 8932 } },
                        { 7241, new[] { 16, 17, 7242, 8839, 8848, 8933 } },
                        { 8848, new[] { 8934, 16, 8854, 8852, 7241, 8839 } },
                        // outer
                        { 8, new[] { 2404, 7246, 7248 } },
                        { 16, new[] { 7242, 17, 8839, 8848, 7241 } },
                        { 17, new[] { 16, 2597, 7240, 7242, 2590 } },
                        { 93, new[] { 7224, 7225, 2401, 2406, 2871 } },
                        { 135, new[] { 136, 137, 138, 2418, 7216 } },
                        { 136, new[] { 135, 137, 138, 2591, 2592 } },
                        { 137, new[] { 136, 2591, 138, 7236, 7237, 7238, 135 } },
                        { 138, new[] { 137, 2418, 2419, 7236, 135 } },
                        { 2401, new[] { 2403, 2402, 2406, 2871, 7243, 7244, 93 } },
                        { 2402, new[] { 2401, 2405, 2406, 7243 } },
                        { 2403, new[] { 2401, 2871, 2404, 7244, 7245, 7246 } },
                        { 2404, new[] { 8, 2403, 7245, 7246, 7248 } },
                        { 2405, new[] { 2402, 2406, 2407, 2409, 7230, 7243, 7247 } },
                        { 2406, new[] { 2401, 2402, 93, 2405, 2407, 7224, 7226 } },
                        { 2407, new[] { 2405, 2406, 2408, 2409, 7223, 7226 } },
                        { 2408, new[] { 7222, 2409, 2412, 2413, 7223, 2407 } },
                        { 2409, new[] { 2408, 2413, 7230, 7231, 2405, 2407 } },
                        { 2410, new[] { 7222, 2411, 2412, 2414, 2417, 7220, 7221 } },
                        { 2411, new[] { 2410, 7231, 2412, 2413, 2414, 7232, 7233 } },
                        { 2412, new[] { 2408, 2410, 2411, 2413, 7222 } },
                        { 2413, new[] { 2408, 2409, 2411, 2412, 7231 } },
                        { 2414, new[] { 2410, 2411, 2415, 2416, 2417, 7233 } },
                        { 2415, new[] { 2414, 7218, 2416, 2417, 2418, 7219 } },
                        { 2416, new[] { 2418, 7233, 7234, 2419, 2414, 2415 } },
                        { 2417, new[] { 7219, 7220, 2410, 2414, 2415 } },
                        { 2418, new[] { 2416, 138, 7216, 7218, 2419, 135, 2415 } },
                        { 2419, new[] { 2416, 138, 2418, 7234, 7235, 7236 } },
                        { 2591, new[] { 136, 137, 7238, 2596, 2592 } },
                        { 2592, new[] { 136, 137, 2591, 2594, 2596 } },
                        { 2594, new[] { 2591, 2592, 2596, 2597, 2598 } },
                        { 2596, new[] { 2592, 2594, 2597, 2598, 7238, 7239, 2591 } },
                        { 2597, new[] { 17, 2596, 2598, 7239, 7240, 2590 } },
                        { 2598, new[] { 2594, 2596, 2597, 2590 } },
                        { 2590, new[] { 17, 2597, 2598 } },
                        { 2871, new[] { 2401, 2403, 93, 7225 } },
                        { 7216, new[] { 7218, 2418, 135 } },
                        { 7218, new[] { 7216, 7219, 2418, 2415 } },
                        { 7219, new[] { 2417, 7218, 7220, 2415 } },
                        { 7220, new[] { 2417, 2410, 7219, 7221 } },
                        { 7221, new[] { 2410, 2412, 2417, 7220, 7222 } },
                        { 7222, new[] { 2408, 2410, 2412, 7221, 7223 } },
                        { 7223, new[] { 2407, 2408, 2412, 7222, 7226 } },
                        { 7224, new[] { 93, 2406, 2407, 7225, 7226 } },
                        { 7225, new[] { 93, 2406, 2871, 7224 } },
                        { 7226, new[] { 2406, 2407, 2408, 7223, 7224 } },
                        { 8837, new[] { 8838, 8849, 8850, 8851, 8853 } },
                        { 8838, new[] { 7248, 8837, 8849, 8850 } },
                        { 8839, new[] { 16, 7241, 8848, 8852, 8854 } },
                        { 8853, new[] { 8837, 8849, 8851, 8852, 8854 } },
                        { 8854, new[] { 8848, 8851, 8852, 8853, 8839 } },
                    };
                case Side.RIGHT:
                    return new Dictionary<int, int[]>
                    {
                        // nipple
                        { 10928, new[] { 18622, 18624, 18643, 18628, 18630, 18644, 18642, 18636, 18632, 18625, 18637, 18623 } },
                        // areola
                        { 18622, new[] { 18623, 18624, 19601 } },
                        { 18624, new[] { 18622, 18643, 19602 } },
                        { 18643, new[] { 18624, 18628, 19597 } },
                        { 18628, new[] { 18630, 18643, 19594 } },
                        { 18630, new[] { 18628, 18644, 19595 } },
                        { 18644, new[] { 18630, 18642, 19591 } },
                        { 18642, new[] { 18636, 18644, 19613 } },
                        { 18636, new[] { 18632, 18642, 19611 } },
                        { 18632, new[] { 18625, 18636, 19607 } },
                        { 18625, new[] { 18632, 18637, 19608 } },
                        { 18637, new[] { 18623, 18625, 19604 } },
                        { 18623, new[] { 18622, 18637, 19599 } },
                        // main ring 1
                        { 19601, new[] { 18622, 19600, 19603, 19523 } },
                        { 19603, new[] { 18622, 18624, 19602, 19601, 19522 } },
                        { 19602, new[] { 18624, 19603, 19606, 19520 } },
                        { 19606, new[] { 18624, 18643, 19602, 19597, 19521 } },
                        { 19597, new[] { 18643, 19606, 19592, 17954 } },
                        { 19592, new[] { 18643, 18628, 19594, 19597, 17952 } },
                        { 19594, new[] { 18628, 19596, 19592, 17951 } },
                        { 19596, new[] { 18628, 18630, 19594, 19595, 17950 } },
                        { 19595, new[] { 18630, 19596, 19593, 17949 } },
                        { 19593, new[] { 18630, 18644, 19591, 19595, 17953 } },
                        { 19591, new[] { 18644, 19593, 19614, 17936 } },
                        { 19614, new[] { 18644, 18642, 19613, 19591, 17937 } },
                        { 19613, new[] { 18642, 19612, 19614, 17938 } },
                        { 19612, new[] { 18642, 18636, 19611, 19613, 17939 } },
                        { 19611, new[] { 18636, 19610, 19612, 17940 } },
                        { 19610, new[] { 18636, 18632, 19611, 19607, 17941 } },
                        { 19607, new[] { 18632, 19609, 19610, 17942 } },
                        { 19609, new[] { 18632, 18625, 19607, 19608, 17943 } },
                        { 19608, new[] { 18625, 19605, 19609, 17944 } },
                        { 19605, new[] { 18625, 18637, 19604, 19608, 17945 } },
                        { 19604, new[] { 18637, 19605, 19598, 17946 } },
                        { 19598, new[] { 18637, 18623, 19599, 19604, 17948 } },
                        { 19599, new[] { 18623, 19598, 19600, 17947 } },
                        { 19600, new[] { 18623, 18622, 19599, 19601, 19519 } },
                        // main ring 2
                        { 19523, new[] { 19510, 19519, 19522, 19524, 19525, 19601 } },
                        { 19522, new[] { 19603, 19520, 19523, 19524, 19525, 19508 } },
                        { 19520, new[] { 19508, 19509, 19521, 19522, 19524, 19602 } },
                        { 19521, new[] { 17954, 19508, 19509, 19520, 19602, 19606 } },
                        { 17954, new[] { 10936, 13234, 17952, 19521, 19597, 19606 } },
                        { 17952, new[] { 10936, 13233, 13234, 17951, 17954, 19592 } },
                        { 17951, new[] { 13231, 13233, 13234, 17950, 17952, 19594 } },
                        { 17950, new[] { 13231, 13232, 13233, 17949, 17951, 19596 } },
                        { 17949, new[] { 19595, 13231, 13232, 17950, 17953, 13235 } },
                        { 17953, new[] { 13232, 13235, 13239, 17936, 17949, 19593 } },
                        { 17936, new[] { 13235, 13239, 13243, 17937, 17953, 19591 } },
                        { 17937, new[] { 19614, 13239, 13241, 13243, 17936, 17938 } },
                        { 17938, new[] { 13241, 13243, 13244, 17937, 17939, 19613 } },
                        { 17939, new[] { 19612, 17938, 13246, 17940, 13241, 13244 } },
                        { 17940, new[] { 13244, 13246, 13249, 17939, 17941, 19611 } },
                        { 17941, new[] { 11066, 13246, 13249, 17940, 17942, 19610 } },
                        { 17942, new[] { 19607, 11065, 11066, 17941, 13249, 17943 } },
                        { 17943, new[] { 11065, 11066, 13409, 17942, 17944, 19609 } },
                        { 17944, new[] { 19608, 11065, 13412, 17943, 17945, 13409 } },
                        { 17945, new[] { 13409, 13412, 13413, 17944, 17946, 19605 } },
                        { 17946, new[] { 10945, 13412, 13413, 17945, 17948, 19604 } },
                        { 17948, new[] { 10944, 10945, 13413, 17946, 17947, 19598 } },
                        { 17947, new[] { 10944, 10945, 17948, 19510, 19519, 19599 } },
                        { 19519, new[] { 19600, 10944, 19525, 19523, 17947, 19510 } },
                        // outer
                        { 10936, new[] { 13234, 17952, 17954 } },
                        { 10944, new[] { 17948, 10945, 19510, 19519, 17947 } },
                        { 10945, new[] { 10944, 13413, 17946, 17948, 13408 } },
                        { 11021, new[] { 17930, 17931, 13231, 13236, 13672 } },
                        { 11063, new[] { 11064, 11065, 11066, 13248, 17923 } },
                        { 11064, new[] { 11063, 11065, 11066, 13409, 13410 } },
                        { 11065, new[] { 11064, 13409, 11066, 17942, 17943, 17944, 11063 } },
                        { 11066, new[] { 11065, 13248, 13249, 17942, 11063 } },
                        { 13231, new[] { 13233, 13232, 13236, 13672, 17949, 17950, 11021 } },
                        { 13232, new[] { 13231, 13235, 13236, 17949 } },
                        { 13233, new[] { 13231, 13672, 13234, 17950, 17951, 17952 } },
                        { 13234, new[] { 10936, 13233, 17951, 17952, 17954 } },
                        { 13235, new[] { 13232, 13236, 13237, 13239, 17936, 17949, 17953 } },
                        { 13236, new[] { 13231, 13232, 11021, 13235, 13237, 17930, 17932 } },
                        { 13237, new[] { 13235, 13236, 13238, 13239, 17929, 17932 } },
                        { 13238, new[] { 17928, 13239, 13242, 13243, 17929, 13237 } },
                        { 13239, new[] { 13238, 13243, 17936, 17937, 13235, 13237 } },
                        { 13240, new[] { 17928, 13241, 13242, 13244, 13247, 17926, 17927 } },
                        { 13241, new[] { 13240, 17937, 13242, 13243, 13244, 17938, 17939 } },
                        { 13242, new[] { 13238, 13240, 13241, 13243, 17928 } },
                        { 13243, new[] { 13238, 13239, 13241, 13242, 17937 } },
                        { 13244, new[] { 13240, 13241, 13245, 13246, 13247, 17939 } },
                        { 13245, new[] { 13244, 17924, 13246, 13247, 13248, 17925 } },
                        { 13246, new[] { 13248, 17939, 17940, 13249, 13244, 13245 } },
                        { 13247, new[] { 17925, 17926, 13240, 13244, 13245 } },
                        { 13248, new[] { 13246, 11066, 17923, 17924, 13249, 11063, 13245 } },
                        { 13249, new[] { 13246, 11066, 13248, 17940, 17941, 17942 } },
                        { 13409, new[] { 11064, 11065, 17944, 13412, 13410 } },
                        { 13410, new[] { 11064, 11065, 13409, 13411, 13412 } },
                        { 13411, new[] { 13409, 13410, 13412, 13413, 134144 } },
                        { 13412, new[] { 13410, 13411, 13413, 13414, 17944, 17945, 13409 } },
                        { 13413, new[] { 10945, 13412, 13414, 17945, 17946, 13408 } },
                        { 13414, new[] { 13411, 13412, 13413, 13408 } },
                        { 13408, new[] { 10945, 13413, 13414 } },
                        { 13672, new[] { 13231, 13233, 11021, 17931 } },
                        { 17923, new[] { 17924, 13248, 11063 } },
                        { 17924, new[] { 17923, 17925, 13248, 13245 } },
                        { 17925, new[] { 13247, 17924, 17926, 13245 } },
                        { 17926, new[] { 13247, 13240, 17925, 17927 } },
                        { 17927, new[] { 13240, 13242, 13247, 17926, 17928 } },
                        { 17928, new[] { 13238, 13240, 13242, 17927, 17929 } },
                        { 17929, new[] { 13237, 13238, 13242, 17928, 17932 } },
                        { 17930, new[] { 11021, 13236, 13237, 17931, 17932 } },
                        { 17931, new[] { 11021, 13236, 13672, 17930 } },
                        { 17932, new[] { 13236, 13237, 13238, 17929, 17930 } },
                        { 19508, new[] { 19509, 19520, 19521, 19522, 19524 } },
                        { 19509, new[] { 17954, 19508, 19520, 19521 } },
                        { 19510, new[] { 10944, 17947, 19519, 19523, 19525 } },
                        { 19524, new[] { 19508, 19520, 19522, 19523, 19525 } },
                        { 19525, new[] { 19519, 19522, 19523, 19524, 19510 } },
                    };
                default: throw new ArgumentException($"Invalid side {side}");
            }
        }
    }
}
